/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/OVTCore.java,v $
  Date:      $Date: 2000/06/12 13:00:43 $
  Version:   $Revision: 1.46 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/* $Id: OVTCore.java,v 1.46 2000/06/12 13:00:43 kono Exp $
 * $Source: /net/nest/stor/devel/ovt2g/ovt/OVTCore.java,v $
 */
package ovt;

import ovt.gui.*;
import ovt.mag.*;
import ovt.util.*;
import ovt.event.*;
import ovt.object.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import vtk.*;

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;

/**
 * This is the class coment for OVTCore
 * @author Mykola Khotyaintsev
 * @version %I% %E%
 * @see ...
 */

public class OVTCore extends OVTObject implements GUIPropertyEditorListener {

  public static final String VERSION = "1.0";
  public static final String settingsFileName = "ovt.conf";
  public static final int DEBUG = 0;

  private vtkRenderWindow renWin = null;
  private vtkRenderer renderer = null;
  private vtkRenderWindow polarRenWin = null;
  private vtkRenderer polarRenderer = null;
  /**
   * @see #setRenderAction(Renderable)
   */
  private Renderable renderAction = null;
  private Renderable polarRenderAction = null;
  private TimeSettings timeSettings;

  public Settings settings = null;
  /** @see #getProperties()
   */
 // protected Properties properties = new Properties();

  public XYZWindow XYZwin;
    //protected int coordSystem = Const.GSM; // XYZ -  GSM, GSE, GEI, ..

  protected static boolean guiPresent = false;

    /** The file, containing all OVT properties.
     *@see #loadProperties()
     */
  //public final static String ovtPropertiesFile ="ovtProperties";

  protected MagProps magProps;
  protected TransCollection transCollection;

  protected CoordinateSystem coordinateSystem;
  
  //  Visual Objects
  protected SunLight sunLight;
  protected Sats sats;
  protected Earth earth;
  protected Axes axes;
  protected Frames frames;
  protected Magnetosphere magnetosphere;
  protected Magnetopause magnetopause;
  protected BowShock bowShock;


  protected boolean isInitialized = false;


  public static String ovtHomeDir = System.getProperty("ovt.home", ".") + File.separator;
  public static String ovtUserDir = System.getProperty("ovt.user", ".") + File.separator;

  public static final String ovtHomePage ="http://ovt.irfu.se/";
  public OVTCore(vtkRenderWindow renWin, vtkRenderer renderer) {
    this.renWin = renWin;
    this.renderer = renderer;
    /*this.polarRenWin = polarRenWin;
    this.polarRenderer = polarRenderer;*/
    Initialize();

    isInitialized = true;
    guiPresent = true;
  }

  public String getName() {
    return "OVT";
  }

  public static String getOVTHomeDir(){
    return ovtHomeDir;
  }

  public static String getUserDir(){
    return ovtUserDir;
  }

  public static String getDocsDir(){
    return getOVTHomeDir() + "docs" + File.separator;
  }

  public static String getImagesDir(){
    return getOVTHomeDir() + "images" + File.separator;
  }

  public static String getUserdataDir(){
    return getUserDir() + "userdata" + File.separator;
  }
  
  public static String getMdataDir(){
    return getOVTHomeDir() + "mdata" + File.separator;
  }

  public static String getConfDir(){
    return getUserDir() + "conf" + File.separator;
  }

  public vtkRenderWindow getRenderWindow()
    { return renWin; }

  public vtkRenderWindow getPolarRenderWindow()
    { return polarRenWin; }

  public vtkRenderer getRenderer()
    { return renderer; }

  public vtkRenderer getPolarRenderer()
    { return polarRenderer; }

  /** Is used to set another Render method, then <CODE>getRenderer.Render()</CODE>.
   * @see #Render()
   * @param renderFunc
   */
public void setRenderAction(Renderable renderAction)
  { this.renderAction = renderAction; }

  /** Is used to set another Render method, then <CODE>getRenderer.Render()</CODE>.
   * @see #Render()
   * @param renderFunc
   */
public void setPolarRenderAction(Renderable renderAction)
  { this.polarRenderAction = renderAction; }




  /** Uses <CODE>getRenderer.Render()</CODE> unless renderAction is specified.
   *@see #setRenderAction(Rendererable)
   */
  public void Render() {
    //setStatus("Rendering...");
    if (renderAction != null) renderAction.Render();
    else getRenderer().Render();
    setStatus("Done.");
  }

  /** Uses <CODE>getRenderer.Render()</CODE> unless renderAction is specified.
   *@see #setRenderAction(Rendererable)
   */
  public void RenderPolar() {
    //System.out.println("Rendering.");
    if (polarRenderAction != null) polarRenderAction.Render();
    else getPolarRenderer().Render();
  }


  /** Render both XYZ and Polar windows */
  public void RenderAll() {
    Render();
    //RenderPolar();
  }

  public Trans getTrans(double mjd) {
    return transCollection.getTrans(mjd);
  }



  /* Load properties from {@link #ovtPropertiesFile }

  public void loadProperties() {
    try {
      FileInputStream in = new FileInputStream(ovtPropertiesFile);
      getProperties().load(in);
    } catch (IOException e) {
      System.out.println("Error loading properties");
      System.err.println(e.toString());
      System.exit(-1);
    }
  }

public void saveProperties() {
  try {
    FileOutputStream out = new FileOutputStream(ovtPropertiesFile);
    getProperties().save(out, "OVT properties file.");
  } catch (IOException e) {
    System.out.println("Error saving properties");
    System.err.println(e.toString());
  }
}*/

/*
public JFrame getVW()
  { return XYZwin; }
*/

  public void Initialize() {
    setName("OVT");
    
    settings = new Settings(getConfDir()+settingsFileName);
    
    magProps = new MagProps(this);
    transCollection = new TransCollection(magProps.getIgrfModel());
    // set time
    timeSettings = new TimeSettings(this);
    // set coordinate system
    coordinateSystem = new CoordinateSystem(this);
    // add sun light
    sunLight = new SunLight(this);
    // set frames
    axes = new Axes(this);
    // set Earth
    earth = new Earth(this);
    // set frames
    frames = new Frames(this);
    // set Bow Shock
    bowShock = new BowShock(this);
    //load sattelites
    sats = new Sats(this);
    //set magnetosphere
    magnetosphere = new Magnetosphere(this);
    //set magnetopause
    magnetopause = new Magnetopause(this);

    //magProps = new ovt.mag.MagProps(this);

    timeSettings.addTimeChangeListener(sunLight);
    timeSettings.addTimeChangeListener(earth);
    timeSettings.addTimeChangeListener(bowShock);
    timeSettings.addTimeChangeListener(sats);
    timeSettings.addTimeChangeListener(magnetosphere);
    timeSettings.addTimeChangeListener(magnetopause);

    coordinateSystem.addCoordinateSystemChangeListener(sunLight);
    coordinateSystem.addCoordinateSystemChangeListener(earth);
    coordinateSystem.addCoordinateSystemChangeListener(bowShock);
    coordinateSystem.addCoordinateSystemChangeListener(sats);
    coordinateSystem.addCoordinateSystemChangeListener(magnetosphere);
    coordinateSystem.addCoordinateSystemChangeListener(magnetopause);
    
    //magProps.addMagPropsChangeListener(bowShock);
    magProps.addMagPropsChangeListener(sats);
    magProps.addMagPropsChangeListener(magnetosphere);
    magProps.addMagPropsChangeListener(magnetopause);
    magProps.addMagPropsChangeListener(bowShock);
    
        /*mps = new Magnetosphere(this, getVW());
  bowShock = new BowShock(this, getVW());
  frames = new Frames(this, getVW());*/
    // set children
    Children children = new Children();
    
    children.put(magProps);  
    children.put(bowShock);
    children.put(magnetopause);
    children.put(magnetosphere);
    children.put(sats);
    children.put(earth);
    children.put(axes);
    children.put(frames);
    setChildren(children);
  }


  public TimeSettings getTimeSettings()
  { return timeSettings; }

  public double getMjd() {
    return getTimeSettings().getCurrentMjd();
  }

  /** Returns current CS*/
  public int getCS() {
    return getCoordinateSystem().getCoordinateSystem();
  }

  /** Returns current CS*/
  public int getPolarCS() {
    return getCoordinateSystem().getPolarCoordinateSystem();
  }

  /** Returns CoordinateSystem*/
  public CoordinateSystem getCoordinateSystem() {
    return coordinateSystem;
  }

  public MagProps getMagProps()
  { return magProps; }

/** Returns true if everything is initialized. Is used for GUI to check if it is necesarry to plot smthng.*/
public boolean isInitialized()
  { return isInitialized; }


public static boolean isGuiPresent()
  { return guiPresent; }

/*
protected void addTimeListenersToTimeCollection() {
  getTimeCollection().addTimeListener(getVW());
  getTimeCollection().addTimeListener(selectedSats);
  getTimeCollection().addTimeListener(mps);
  getTimeCollection().addTimeListener(bowShock);
}*/


  public Children getChildren() {
    if (children == null) {

    }
    return children;
  }


  // this method is a part of GUIPropertyEditorListener
  // this listener is added to all editors, to render after
  // user changes some paramiters
  public void editingFinished(GUIPropertyEditorEvent evt) {
    RenderAll();
  }

  /** Method to be used when iforming user about the exception occured.
   *When in GUI produces a popup window with <I>OVT Exception</I>
   *as window title and <CODE>e.toString()</CODE> as message.
   * @param e Exception
   */
  public void sendErrorMessage(Exception e) {
    String msg, msghead;
    msghead = "OVT Exception";
    msg = e.toString();
    if (isGuiPresent() == true){
      javax.swing.JOptionPane.showMessageDialog(null, msg, msghead,
      javax.swing.JOptionPane.ERROR_MESSAGE);
    } else {
      System.out.println(msg);
    }
  }
  /** Method to be used when informing user about the error occured
   *When in GUI produces a popup window with <CODE>msghead</CODE>
   *as window title and <CODE>msg</CODE> as message.
   * @param msg Error message
   * @param msghead Error title
   */
  public void sendErrorMessage(String msg,String msghead) {
    if (isGuiPresent() == true){
      javax.swing.JOptionPane.showMessageDialog(null, msg, msghead,
      javax.swing.JOptionPane.ERROR_MESSAGE);
    } else {
      System.out.println(msghead + ":" + msg);
    }
  }
    /** Method to be used when informing user about the warning
     *When in GUI produces a popup window with <CODE>msghead</CODE>
     *as window title and <CODE>msg</CODE> as message.
     * @param msg Warning message
     * @param msghead Warning title
     */
    public void sendWarningMessage(String msg,String msghead) {
      if (isGuiPresent() == true){
        javax.swing.JOptionPane.showMessageDialog(null, msg, msghead,
        javax.swing.JOptionPane.ERROR_MESSAGE);
      } else {
        System.out.println(msghead + ":" + msg);
      }
    }



    public static void setStatus(String statusMessage) {
      if (isGuiPresent() == true){
        XYZWindow.setStatus(statusMessage);
      } else {
        System.out.println("status : " + statusMessage);
      }
    }
      // for JNI methods
  static {
    System.loadLibrary("ovt2g");
  }

  /** Detect OS type and return true if OS=windows */
  public static boolean isUnderWindows() {
    String osName = System.getProperty("os.name");
    if (osName.toLowerCase().indexOf("win") != -1) return true;
    return false;
  }


  public void setXYZWindow(XYZWindow XYZwin) {
    this.XYZwin = XYZwin;
  }

}
