/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/XYZWindow.java,v $
  Date:      $Date: 2000/06/13 21:45:32 $
  Version:   $Revision: 1.29 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt;

import ovt.*;
import ovt.gui.*;
import ovt.util.*;
import ovt.event.*;
import ovt.object.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import vtk.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.IOException;

import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.event.*;



public class XYZWindow extends JFrame implements ActionListener {

  protected OVTCore core;
  //protected PolarWindow polarWindow;
  protected JWindow splashWindow;

  // VTK stuff
  protected vtkPanel renPanel;
  protected vtkRenderer ren;
  // GUI
  protected static StatusLine statusLine = new StatusLine();
  protected TreePanel treePanel;
  public boolean windowResizable = true;
  protected XYZMenuBar menuBar;
  protected JSplitPane splitPane;
  protected XYZToolBar toolBar;
  //protected HelpWindow helpWindow;
  protected HTMLBrowser htmlBrowser;
  
  public XYZWindow() {
    super("Orbit Visualization Tool (version" 
    + OVTCore.VERSION + ")");
    setIconImage (Toolkit.getDefaultToolkit().getImage(core.getImagesDir()+"ovt.gif"));
    if(System.getProperty("os.name").equalsIgnoreCase("Windows 95")) windowResizable = false;
    // show splashscreen
    splashWindow = new JWindow();
    JLabel label = new JLabel(new ImageIcon(core.getImagesDir()+"splash.gif"));
    label.setBorder(BorderFactory.createRaisedBevelBorder());
    splashWindow.getContentPane().add(label, BorderLayout.CENTER);
    splashWindow.pack();
    // senter splash window
    Dimension scrnSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension windowSize = splashWindow.getSize();
    splashWindow.setLocation(scrnSize.width/2 - windowSize.width/2,
                 scrnSize.height/2 - windowSize.height/2);

    splashWindow.show();


    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });

//------- create the vtkPanel ----------

    addNotify();
    renPanel = new vtkPanel();
//------- create the OVTCore ----------

    core = new OVTCore(renPanel.getRenderWindow(), renPanel.getRenderer());

    int sizeX = 400;
    int sizeY = 400;
    try{
      //core.settings.list(System.out);
      sizeX = new Integer(core.settings.getProperty("XYZWindow.size.x")).intValue();
      sizeY = new Integer(core.settings.getProperty("XYZWindow.size.y")).intValue();
    }catch(NumberFormatException e){
      System.out.println("Very-very strange !");
    }
    renPanel.setSize(sizeX, sizeY);
    //renPanel.re
    // set the renderer
    ren = renPanel.getRenderer();
    ren.SetBackground(0.9,0.9,0.9);
    //ren.SetBackground(0, 0, 0);

    addOriginActor();
//------- create the PolarWindow -------
    //polarWindow = new PolarWindow();

    core.setXYZWindow(this);

    core.setRenderAction(new Renderable() {
                            public void Render() {
                                renPanel.Render();
                            }
                          });
    //System.out.println("core done.");
    /*
    core.setPolarRenderAction(new Renderable() {
                            public void Render() {
                                polarWindow.Render();
                            }
                          });
//------- setup the PolarWindow -------
    polarWindow.setCore(core);
    polarWindow.start();*/

//------- set up the menu -------------

    JPopupMenu.setDefaultLightWeightPopupEnabled(false);

    menuBar = new XYZMenuBar(getCore(), this);
    setJMenuBar(menuBar);

    Container contentPane = getContentPane();
    contentPane.setLayout(new BorderLayout());
  
//-------- create the tree panel -----
    treePanel = new TreePanel(getCore());
    Dimension ddd = new Dimension(200, 400);
    treePanel.setPreferredSize(ddd);
    /*if (windowResizable) {
      treePanel.setMinimumSize(ddd);
      treePanel.setMaximumSize(ddd);
    }*/
  
//--------Create a split pane with the two scroll panes in it
    splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,windowResizable);

    splitPane.setLeftComponent(treePanel);
    splitPane.setRightComponent(renPanel);
    splitPane.setOneTouchExpandable(windowResizable);
    //splitPane.res
  //Provide minimum sizes for the two components in the split pane
    Dimension minimumSize = new Dimension(200, 400);
  //renPanel.setMinimumSize(minimumSize);
  //treePanel.setMinimumSize(minimumSize);

  //Set the initial location and size of the divider
  //splitPane.setDividerLocation(400);
    splitPane.setDividerSize(10);

  //Provide a preferred size for the split pane
  //splitPane.setPreferredSize(new Dimension(500, 400));
    // add a split pane
    contentPane.add(splitPane, BorderLayout.CENTER);
  
    // add main ToolBar
    toolBar = new XYZToolBar(getCore(), this);
    contentPane.add(toolBar, BorderLayout.SOUTH);
    
    // add Status Line
    contentPane.add(statusLine, BorderLayout.NORTH);
    
    vtkCamera camera = ren.GetActiveCamera();
    //camera.Zoom(0.05);
    //camera.ParallelProjectionOn(); 
    
    camera.SetViewUp (0, 0, 1);
    camera.SetPosition (10, 10, 1);
    camera.SetFocalPoint (0, 0, 0);
    //System.out.println("Setting clipping range...");
    camera.SetClippingRange(0.5, 200);
    camera.ComputeViewPlaneNormal();
    double[] cr = camera.GetClippingRange();
    //System.out.println("Clipping range: "+cr[0]+"\t"+cr[1]);


    /*
  //camera.SetParallelScale(500);
  System.out.println("Parallel scale: "+camera.GetParallelScale());
  
  //camera.SetParallelScale(10);
  System.out.println("Thickness: "+camera.GetThickness());
  
  
  

  cr = camera.GetClippingRange();
  System.out.println("Clipping range: "+cr[0]+"\t"+cr[1]);
  
  System.out.println("Thickness: "+camera.GetThickness());


  //double[] fp = camera.GetFocalPoint();
  //System.out.println("Focal point: "+fp[0]+"\t"+fp[1]+"\t"+fp[2]);

*/
//  enableEvents(AWTEvent.WINDOW_EVENT_MASK);

        // create Help Window
    htmlBrowser = new HTMLBrowser(core);
}

  public void start() {
    //refreshGUI();
    pack();
    Dimension scrnSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension windowSize = getSize();
    /*setLocation(scrnSize.width/2 - windowSize.width/2, 
                 scrnSize.height/2 - windowSize.height/2);*/
    setLocation(100, scrnSize.height/2 - windowSize.height/2);
    splashWindow.dispose();
    this.setResizable(windowResizable);
    setVisible(true);
    //polarWindow.setVisible(true);
    //getCore().start(); 
    
  }

  public OVTCore getCore()
  { return core; }

  public void actionPerformed(ActionEvent e) {
  }


  public void Render() 
  { renPanel.Render(); }

  public vtkRenderer getRenderer() { 
    return renPanel.getRenderer(); 
  }
   
  public vtkRenderWindow getRenderWindow() { 
    return renPanel.getRenderWindow(); 
  }


public void quit() {
  //if (quitConfirmed()) {System.exit(0);}
  System.exit(0);
}


  public static void setStatus(String statusMessage) {
    statusLine.setStatus(statusMessage);
  }

/**
 * Returns <code>true</code> if user pressed <code>Yes</code> 
 * else <code>false</code>.
 * @param none
 * @return see upp!
 * @see javax.swing.JOptionPane#showOptionDialog
 */

  private boolean quitConfirmed() {
        Object[] options = {"Yes", "No"};
        int n = JOptionPane.showOptionDialog(this,"Do you really want to exit?", "Exit", JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,"No");
        if (n == JOptionPane.YES_OPTION) {
            return true;
        } else {
            return false;
        }
  }

  /** 
   * Main method. Here we launch OVT
   */

  public static void main(String[] arg) {
    XYZWindow XYZwin = new XYZWindow();
    XYZwin.start();
  }

  public HTMLBrowser getHTMLBrowser() {
    return htmlBrowser;
  }

  protected void addOriginActor() {
    vtkVectorText atext = new vtkVectorText();
        atext.SetText(". (0, 0, 0)");
    vtkPolyDataMapper mapper = new vtkPolyDataMapper();
        mapper.SetInput(atext.GetOutput());
    vtkFollower actor = new vtkFollower();
        actor.SetMapper(mapper);
        actor.SetScale(0.02, 0.02, 0.02);
        actor.AddPosition(0, 0, 0);
        actor.SetCamera(getRenderer().GetActiveCamera());
        actor.GetProperty().SetColor(0, 0, 0);
        getRenderer().AddActor(actor);
  }
  
   
  public void setRenWinSize(int width, int height) {
    renPanel.setSize(width, height);
    treePanel.setSize((int)treePanel.getSize().width,height+50);
    pack();
  }
  public boolean isWindowResizable() {
    return windowResizable;
  }
}
