/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/datatype/DumpRecord.java,v $
  Date:      $Date: 2000/06/14 13:50:01 $
  Version:   $Revision: 1.5 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.datatype;

import java.lang.*;
import java.io.*;

/** 
 *
 * @author  kono
 * @version 
 */
public class DumpRecord extends Object{
  
  public static final byte numOfFields = 16;
  
  public static final byte TIME = 0;
  public static final byte POS_LABEL = 1;
  public static final byte POSX = 2;
  public static final byte POSY = 3;
  public static final byte POSZ = 4;
  public static final byte RAD = 5;
  public static final byte VEL = 6;
  public static final byte FOOT_LABEL= 7;
  public static final byte LAT1 = 8;
  public static final byte LON1 = 9;
  public static final byte LAT2 = 10;
  public static final byte LON2 = 11;
  public static final byte MP_DIST_LABEL = 12;
  public static final byte MP_DIST = 13;
  public static final byte SPIN_LABEL = 14;
  public static final byte SPIN = 15;
  
  public static final String strMP_DIST_LABEL = "Distance to magnetopause";
  public static final String strSPIN_LABEL = "Spin axis angles (degrees)";
  
  private String[] fields = new String[numOfFields];
  
  public DumpRecord(){
    for(int i=0;i<numOfFields;++i)
      switch(i){
        case MP_DIST_LABEL: setField(i, new String(strMP_DIST_LABEL)); continue;
        case SPIN_LABEL: setField(i, new String(strSPIN_LABEL)); continue;
        default: setField(i, " ");
      }
  }
  
  public DumpRecord(DumpRecord source){
    for(byte i=0;i<numOfFields;++i)
      this.fields[i] = new String(source.fields[i]);
  }
  
  public String getField(int id){
    if(id<0 || id>=numOfFields)
      return "";
    else
      return new String(this.fields[id]);
  }
  
  public void setField(int id,String value){
    if(id>=0 && id<numOfFields && value.length()!=0)
      fields[id] = new String(value);
  }
  
  public Object clone(){
    return new DumpRecord(this);
  }
  
  public void print(){
    System.out.println("Printing DumpRecord:");
    for(int i=0;i<this.numOfFields;++i)
      System.out.println(getField(i));
  }
  
}
