/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/datatype/TimeSet.java,v $
  Date:      $Date: 2000/04/11 15:42:01 $
  Version:   $Revision: 1.5 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 * TimeSet.java
 *
 * Created on February 28, 2000, 11:26 AM
 */
 
package ovt.datatype;

/** 
 *
 * @author  mykola
 * @version 
 */
public class TimeSet {
  
  private double startMjd;
  private double stopMjd;
  private double stepMjd;
  private double currentMjd;
  
  private boolean startMjdReady = false;
  private boolean stopMjdReady = false;
  private boolean stepMjdReady = false;
  
  
  /** Creates new TimeSet */
  public TimeSet() {}
  
  /** Creates new TimeSet */
  public TimeSet(double startMjd, double stopMjd, double stepMjd) {
    setStartMjd(startMjd);
    setStopMjd(stopMjd);
    setStepMjd(stepMjd);
    currentMjd = startMjd;
  }
  
  public boolean isReady() {
    return (startMjdReady && stopMjdReady && stepMjdReady);
  }
  
  public void clear() {
    startMjdReady = false;
    stopMjdReady = false;
    stepMjdReady = false;;
  }
  
  public void setStartMjd(double value) { 
    startMjd = value;
    startMjdReady = true;
  }
  
  public void setStopMjd(double value) { 
    stopMjd = value;
    stopMjdReady = true;
  }

  public void setStepMjd(double value) { 
    stepMjd = value;
    stepMjdReady = true;
  }
  
  public double getStartMjd() {
    return startMjd;
  }
  
  public double getStopMjd() {
    return stopMjd;
  }
  
  public double getStepMjd() {
    return stepMjd;
  }

  public void setCurrentMjd(double value) { 
    currentMjd = value;
  }
  
  public double getCurrentMjd() {
    return currentMjd;
  }
  
  public double[] getTimeMap() {
    int nOfValues = (int)((stopMjd - startMjd)/stepMjd) + 1;
    double[] res = new double[nOfValues];
    double current = startMjd;
    for (int i=0; i<nOfValues; i++) {
      res[i] = current;
      current+=stepMjd;
    }
    return res;
  }
  
  public Object clone() {
    TimeSet ts = new TimeSet(startMjd, stopMjd, stepMjd);
    return ts;
  }
  
  public String toString() {
    return "start="+Time.toString(startMjd)+
           " stop="+Time.toString(stopMjd)+
           " step="+MinutesAndSeconds.toString(stepMjd);
  }
}