/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/gui/TreePanel.java,v $
  Date:      $Date: 2000/04/11 17:52:33 $
  Version:   $Revision: 1.8 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.gui;

import ovt.*;
import ovt.beans.*;
import ovt.object.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class TreePanel extends JScrollPane implements ActionListener {
	protected OVTCore core;
	protected JTree tree = null;
	protected DefaultTreeModel treeModel;
	protected OVTNode rootNode = null;
	/** True if the tree consists only from one Root node */
	protected boolean isEmpty = true;
	
  public TreePanel(OVTCore core) {
    super();
    this.core = core;
        
        //Add listener to components that can bring up popup menus.
    MouseListener popupListener = new PopupListener();
    //Create the nodes.
    //System.out.println("Creating nodes");  
    rootNode = new OVTNode(core, this);
    //System.out.println("finished reating nodes");
    
    treeModel = new DefaultTreeModel((DefaultMutableTreeNode)rootNode);
    tree = new JTree(treeModel);
    tree.setCellRenderer(new OVTTreeCellRenderer());
    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        //Listen for when the selection changes.
    tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();

                if (node == null) return;

                Object nodeInfo = node.getUserObject();
                if (node.isLeaf()) {
                	String book = nodeInfo.toString();
                    //BookInfo book = (BookInfo)nodeInfo;
                    //displayURL(book.bookURL);
                    
                    //System.out.print("Leaf :  \n    ");
                    
                } else {
                    //displayURL(helpURL); 
                    //System.out.print("Node : ");
                }
                //System.out.println(nodeInfo.toString());
                
            }
        });
		
        //tree.putClientProperty("JTree.lineStyle", lineStyle);
    tree.putClientProperty("JTree.lineStyle", "Angled");
    tree.addMouseListener(popupListener);
        //setColumnHeaderView(tree);
    setViewportView(tree);
  }

  public OVTCore getCore()
	{ return core; }

  /** notifies the tree model listeners that <CODE>node</CODE> has changed */        
  public void nodeChanged(OVTNode node) {
    treeModel.nodeChanged(node);
  }
        
  public void actionPerformed(ActionEvent e) {
        JMenuItem source = (JMenuItem)(e.getSource());
        String s = "Action event detected";
        System.out.println(s);
  }


class PopupListener extends MouseAdapter {
  
  public void mouseClicked(MouseEvent e) {
    int selRow = tree.getRowForLocation(e.getX(), e.getY());
    TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
    DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();
    if (node == null) return;
    Object obj = node.getUserObject();
    if (obj instanceof String) return;
    
    if (selRow != -1) {
      if(e.getClickCount() == 1) {
                 //mySingleClick(selRow, selPath);
        //System.out.println("One click ");
			
      } else if(e.getClickCount() == 2) {
				//---- DOUBLE CLICK ------
        //System.out.println("Double click ");
        //treeMember.doubleClicked();
      }	
    }
  }
 
  public void mousePressed(MouseEvent e) {
            maybeShowPopup(e);
  }

  public void mouseReleased(MouseEvent e) {
            maybeShowPopup(e);
  }

  private void maybeShowPopup(MouseEvent e) {
    int selRow = tree.getRowForLocation(e.getX(), e.getY());
    TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());

    DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();

    if (node == null) return;

    Object obj = node.getUserObject();

    if (selRow != -1) {
      if  (e.isPopupTrigger()) {
         		
                // select the item, when pressing right mouse
                // button 
                /*if (!tree.getLeadSelectionPath().equals(selPath)) {
                	tree.setSelectionPath(selPath); 
                	
				}*/
        try {
          JPopupMenu popupMenu = getPopupMenu((DescriptorsSource)obj);
	  if (popupMenu != null ) popupMenu.show(e.getComponent(), e.getX(), e.getY());
        } catch (ClassCastException e2) {
          System.out.println("This object has no popup menu.");
        }
      } 
    }
  }

}

  public JPopupMenu getPopupMenu(DescriptorsSource source) {
    JPopupMenu popupMenu = null;
    Descriptors descriptors = source.getDescriptors();
    if (descriptors == null) return null;
    Enumeration e = descriptors.elements();
    BasicPropertyDescriptor pd;
    GUIPropertyEditor menuItemsSource;
    JMenuItem[] mItems;
    int i;
    while (e.hasMoreElements()) {
      pd = (BasicPropertyDescriptor)(e.nextElement());
      //System.out.println("Property=" + pd.getName());
      
      if (pd.isMenuAccessible()) {
        
        try {
          menuItemsSource = (GUIPropertyEditor)(pd.getPropertyEditor());
        
        // One should render after user changes any paramiter by means of editor
        // core will render, when recieves event from property editor.
          if (!menuItemsSource.hasListener((GUIPropertyEditorListener)getCore()))
            menuItemsSource.addGUIPropertyEditorListener((GUIPropertyEditorListener)getCore());
        
          mItems = null;
          mItems = menuItemsSource.getMenuItems();
          if (mItems != null) {
            for (i=0; i<mItems.length; i++) {
              if (popupMenu == null) popupMenu = new JPopupMenu();
              popupMenu.add(mItems[i]);
            }
            if (e.hasMoreElements()) popupMenu.add(new JSeparator());
          }
        } catch (ClassCastException e2) {
          System.out.println("this editor has no menu items.");
          return null;
        }
      }
    }
    return popupMenu;
  }

}