/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/gui/XYZToolBar.java,v $
  Date:      $Date: 2000/06/13 21:46:04 $
  Version:   $Revision: 1.7 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.gui;

import ovt.*;
import ovt.gui.*;
import ovt.util.*;
import ovt.beans.*;
import ovt.object.*;
import ovt.object.editor.*;
import ovt.datatype.*;


import vtk.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

public class XYZToolBar extends JToolBar implements ActionListener {

  protected OVTCore core;
  protected XYZWindow XYZwin;
  //protected CoordSystemComboBox csComboBox;


  public XYZToolBar(OVTCore core, XYZWindow xyzwin) {
    super();
    XYZwin = xyzwin;
    this.core = core;
    JPopupMenu.setDefaultLightWeightPopupEnabled(false);
    setMargin(new Insets(1, 1, 1, 1));
  
    addButtons();
    addSeparator();
    addCoordinateSystemComboBox();
    addPolarCoordinateSystemComboBox();
    addSeparator();
    addVCRButtons();
    addSeparator();
	
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	
	//addSeparator();

	// Shirt camera controls here
	
	JButton button = new JButton("vX");
	button.setToolTipText("Veiw from X axis");
	//button.addActionListener(XYZwin.cameraPanel);
	button.setEnabled(false);
	//add(button);

	button = new JButton("vY");
	button.setToolTipText("Veiw from Y axis");
	//button.addActionListener(XYZwin.cameraPanel);
	button.setEnabled(false);
	//add(button);

	button = new JButton("vZ");
	button.setToolTipText("Veiw from Z axis");
	//button.addActionListener(XYZwin.cameraPanel);
	button.setEnabled(false);
	//add(button);
	
	//addSeparator();
	
	button = new JButton("Camera");
	button.setToolTipText("Show camera controls");
	//button.addActionListener(XYZwin.cameraPanel);
	button.setEnabled(false);
	//add(button);
	

}

public OVTCore getCore()
	{ return core; }

public XYZWindow getVW()
	{ return XYZwin; }

  protected void addButtons() {

	JButton button = null;

        String imgDir = OVTCore.getImagesDir();
        
	button = new JButton(new ImageIcon(imgDir+"Save.gif"));
	button.setName("Export");
	button.setToolTipText("Export image");
	button.addActionListener(this);
	button.setAlignmentY(0.5f);
        add(button);
        
        
	button = new JButton(new ImageIcon(imgDir+"Print.gif"));
	button.setName("Print");
	button.setToolTipText("Print");
	button.addActionListener(this);
        button.setAlignmentY(0.5f);
        if ( !XYZwin.windowResizable ){ 
          button.setEnabled(false);
        }
	add(button);
        
	addSeparator();


	button = new JButton(new ImageIcon(imgDir+"Clock.gif"));
	button.setName("Time");
	button.setToolTipText("Change sattelite's time properties");
	button.addActionListener(this);
        button.setAlignmentY(0.5f);
	add(button);
  }

  
  protected void addCoordinateSystemComboBox() {
    BasicPropertyDescriptor  pd = getCore().getCoordinateSystem().getDescriptors().getDescriptor("coordinateSystem");
    Component comp = ((ComponentPropertyEditor)pd.getPropertyEditor()).getComponent();
    add(comp);
  }

  protected void addPolarCoordinateSystemComboBox() {
    BasicPropertyDescriptor  pd = getCore().getCoordinateSystem().getDescriptors().getDescriptor("polarCoordinateSystem");
    Component comp = ((ComponentPropertyEditor)pd.getPropertyEditor()).getComponent();
    add(comp);
  }

  
  protected void addVCRButtons() {

  BasicPropertyDescriptor  pd = getCore().getTimeSettings().getDescriptors().getDescriptor("currentMjd");
  Component[] comps = ((CurrentMjdEditor)(pd.getPropertyEditor())).getComponents();
  for (int i=0; i<comps.length; i++) {
    add(comps[i]);
  }
}


  public void actionPerformed(ActionEvent e) {
	
	Component  comp = (Component)e.getSource();
	String compName = comp.getName();
	if (compName.equals("Export")) ImageOperations.exportImageDialog(getVW(), getVW().getRenderWindow());
	
	if (compName.equals("Print")) ImageOperations.print(getVW(), getVW().getRenderWindow());
	
        if (compName.equals("Time")) {
          BasicPropertyDescriptor pd = getCore().getTimeSettings().getDescriptors().getDescriptor("time");
          ((WindowPropertyEditor)(pd.getPropertyEditor())).setWindowVisible(true);
        }
  }

}
