/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/mag/MagProps.java,v $
  Date:      $Date: 2000/06/13 11:34:06 $
  Version:   $Revision: 1.23 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.mag;


import ovt.*;
import ovt.beans.*;
import ovt.util.*;
import ovt.event.*;
import ovt.interfaces.*;
import ovt.mag.model.*;
import ovt.object.*;

import java.beans.*;
import java.util.*;
import javax.swing.*;

/** Class contains time independent magnetic field properties
 * properties.
 */
public class MagProps extends OVTObject implements MagModel {


/** @see #useExperimentalActivity(boolean) */
protected static boolean useExperimentalActivity = false;

private ovt.OVTCore core;

protected static MagActivity userActivity = new MagActivity();

/** Minimum absolut value of magnetic field */
public static double BMIN = 5.5836;
/** Maximum absolut value of magnetic field */
public static double BMAX = 58044;
/** magnetic moment of the earth for igrf1985 model */
public static final double DIPMOM = -30483.03;


public static final int NOMODEL  = 0;
public static final int DIPOLE   = -10;
public static final int IGRF   =  10;
public static final int T87 = 87;
public static final int T89 = 89;
public static final int T96 = 96;

/* model = -21.0 ....-25.0      (dipole + tsyganenko 89) <BR>
 *                -11.0 ... -15.0      (dipole + tsyganenko 87)<BR>
 *                -10.0                (dipole)<BR>
 *                 10.0                (igrf)<BR>
 *                 11.0 ...  15.0      (igrf +   tsyganenko 87)<BR>
 *                 21.0 ...  25.0      (igrf + tsyganenko 89)<BR>
 */

//public static double model=-10.0;

/**  arbitrary factor from <code>0.5</code> to <code>1.5</code>
 *  Field = internalField  +  ModelFactor * externalFiels
 *  @see ovt.calc.MagPack#magbv(double[])
 */

public static double modelFactor=1.0;

public static double mSub=11.0;
public static double bSub=13.5;

/** Minimum distance in the tail. Should be negative. */
public static double xlim = -30;

/** Altitude (km) for footprint tracing */
public static double alt = 100;
/** Altitude (RE) for footprint tracing */
protected double footprintAltitude = 100./Const.RE;

/** Holds (ModelType, Model) pairs. Hashtable is */
protected Hashtable models = new Hashtable();

/** Utility field used by constrained properties. */
private java.beans.VetoableChangeSupport vetoableChangeSupport = new java.beans.VetoableChangeSupport (this);
/** Utility field used by bound properties. */
private OVTPropertyChangeSupport propertyChangeSupport = new OVTPropertyChangeSupport (this);

private MagPropsChangeSupport magPropsChangeSupport = new MagPropsChangeSupport(this);

/** Holds value of property modelTypes. */
private ModelTypes modelTypes = new ModelTypes(IGRF, T87);

protected AbstractMagModel internalModel = null;
protected AbstractMagModel externalModel = null;

/* Magnetic Activity */
public static String KPINDEX = "KPIndex";
public static String IMF = "IMF";
public static String SWP = "SWP";
public static String DSTINDEX = "DSTIndex";
public static String MACHNUMBER = "MachNumber";
public static String MPCLIP = "MP Clipping";  //Added by kono

protected MagActivityDataModel kpindex = null;
protected MagActivityDataModel imf = null;
protected MagActivityDataModel swp = null;
protected MagActivityDataModel dstindex = null;
protected MagActivityDataModel machnumber = null;

public static double KPINDEX_DEFAULT = 0;
public static double[] IMF_DEFAULT = {0,0,0};
public static double SWP_DEFAULT = 1.8;
public static double DSTINDEX_DEFAULT = -40;
public static double MACHNUMBER_DEFAULT = 5.4;

public static double KPINDEX_LAST = 0;
public static double[] IMF_LAST = {0,0,0};
public static double SWP_LAST = 1.8;
public static double DSTINDEX_LAST = -40;
public static double MACHNUMBER_LAST = 5.4;

public MagActivityDataEditor kpindexEditor = null;
public MagActivityDataEditor imfEditor = null;
public MagActivityDataEditor swpEditor = null;
public MagActivityDataEditor dstindexEditor = null;
public MagActivityDataEditor machnumberEditor = null;

private MagPropsEditor magPropsEditor= null;

/** Holds value of property editorVisible. */
private boolean editorVisible;

/** Magnetopause clipping. */
private static boolean mpClipping = true; //added by kono

/** Creates new magProperties. */
public MagProps(OVTCore core) {
  super("Mag Models");
  setIcon(new ImageIcon(OVTCore.getImagesDir()+"magnet.gif"));
  this.core = core;
  try {
    kpindex = new MagActivityDataModel(KPINDEX,0,9);
    kpindexEditor = new MagActivityDataEditor(kpindex);
  } catch (Exception e) {
    core.sendWarningMessage(e.toString()+"\n will use the default value",
    "KPIndex initialization error");
  }
  try {
    imf = new MagActivityDataModel(IMF,-50,50);
    imfEditor = new MagActivityDataEditor(imf);
  } catch (Exception e) {
    core.sendWarningMessage(e.toString()+"\n will use the default value",
    "IMF initialization error");
  }
  try {
    swp = new MagActivityDataModel(SWP,0,50);
    swpEditor = new MagActivityDataEditor(swp);
  } catch (Exception e) {
    core.sendWarningMessage(e.toString()+"\n will use the default value",
    "SWP initialization error");
  }
  try {
    dstindex = new MagActivityDataModel(DSTINDEX,-500, 50);
    dstindexEditor = new MagActivityDataEditor(dstindex);
  } catch (Exception e) {
    core.sendWarningMessage(e.toString()+"\n will use the default value",
    "DSTIndex initialization error");
  }
  try {
    machnumber = new MagActivityDataModel(MACHNUMBER,1, 9);
    machnumberEditor = new MagActivityDataEditor(machnumber);
  } catch (Exception e) {
    core.sendWarningMessage(e.toString()+"\n will use the default value",
    "MachNumber initialization error");
  }
  magPropsEditor = new MagPropsEditor(this);
  editorVisible = false;
}

public OVTCore getCore() {
  return core;
}

public Trans getTrans(double mjd) {
  return getCore().getTrans(mjd);
}

/*public String getName(){
  return "ovt.mag.MagProps";
}*/

public static double getMaxB()
  { return BMAX; }

public static double getMinB()
  { return BMIN; }

public double getModelFactor() {
  return modelFactor;
}

public AbstractMagModel getModel(int modelType) {
  AbstractMagModel model = (AbstractMagModel)models.get(new Integer(modelType));
  if (model == null) {
    // there is no model of modelType
    // create it
    switch (modelType) {
      case NOMODEL: model = new NullModel(this); break;
      case DIPOLE : model = new DipoleModel(this); break;
      case IGRF   : model = new IgrfModel(this); break;
      case T87    : model = new Tsyganenko87(this); break;
      case T89    : model = new Tsyganenko89(this); break;
      case T96    : model = new Tsyganenko96(this); break;
      default: throw new IllegalArgumentException("Invalid model type :" + modelType);
    }
    models.put(new Integer(modelType), model);
  }
  return model;
}

public IgrfModel getIgrfModel() {
  return (IgrfModel)getModel(IGRF);
}

//added by kono
public static void setMPClipping(boolean mode){
  mpClipping = mode;
}

public static boolean isMPClipping(){
  return mpClipping;
}

public AbstractMagModel getInternalModel() {
  if (internalModel == null) internalModel = getModel(getInternalModelType());
  return internalModel;
}

public AbstractMagModel getExternalModel() {
  if (externalModel == null) externalModel = getModel(getExternalModelType());
  return externalModel;
}



/** returns magnetic field vector
 * @param gsm Coordinate in GSM
 * @param mjd time
 * @param internalModel internal model type
 * @param externalModel external model type
 * @return magnetic field vector in nT
 */
public double[] bv(double[] gsm, double mjd, int internalModel, int externalModel) {
  double[] res = new double[3];
  double[] intbv = getModel(internalModel).bv(gsm, mjd);
  double[] extbv = getModel(externalModel).bv(gsm, mjd);
  for (int i=0; i<3; i++) res[i] = intbv[i] + getModelFactor() * extbv[i];
  return res;
}

/** Returns magnetic field vector using current internal and external field models.
 * @param gsm point in GSM
 * @param mjd time
 * @return magnetic field vector in nT
 */
public double[] bv(double[] gsm,double mjd) {
  return bv(gsm, mjd, getInternalModelType(), getExternalModelType());
}

/** Add a PropertyChangeListener to the listener list.
 * @param l The listener to add.
 */
public void addPropertyChangeListener(java.beans.PropertyChangeListener l) {
  propertyChangeSupport.addPropertyChangeListener (l);
}
/** Removes a PropertyChangeListener from the listener list.
 * @param l The listener to remove.
 */
public void removePropertyChangeListener(java.beans.PropertyChangeListener l) {
  propertyChangeSupport.removePropertyChangeListener (l);
}
/** Add a VetoableChangeListener to the listener list.
 * @param l The listener to add.
 */
public void addVetoableChangeListener(java.beans.VetoableChangeListener l) {
  vetoableChangeSupport.addVetoableChangeListener (l);
}
/** Removes a VetoableChangeListener from the listener list.
 * @param l The listener to remove.
 */
public void removeVetoableChangeListener(java.beans.VetoableChangeListener l) {
  vetoableChangeSupport.removeVetoableChangeListener (l);
}

public void addMagPropsChangeListener (MagPropsChangeListener listener) {
  magPropsChangeSupport.addMagPropsChangeListener (listener);
}

public void removeMagPropsChangeListener (MagPropsChangeListener listener) {
  magPropsChangeSupport.removeMagPropsChangeListener (listener);
}


/** Getter for property modelTypes.
 * @return Value of property modelTypes.
 */
public ModelTypes getModelTypes() {
  return modelTypes;
}
/** Setter for property modelTypes.
 * @param modelTypes New value of property modelTypes.
 *
 * @throws PropertyVetoException
 */
public void setModelTypes(ModelTypes modelTypes) throws java.beans.PropertyVetoException {
  ModelTypes oldModelTypes = this.modelTypes;
  // check modelTypes for being correct

  // internal model type
  int type = modelTypes.getInternal();
  if (type!=NOMODEL && type!=DIPOLE && type!=IGRF)
      throw new IllegalArgumentException("Invalid internal field model type");

  // external model type
  type = modelTypes.getExternal();
  if (type!=NOMODEL && type!=T87 && type!=T89 && type!=T96)
      throw new IllegalArgumentException("Invalid external field model type");

  vetoableChangeSupport.fireVetoableChange("modelTypes", oldModelTypes, modelTypes);
  this.modelTypes = modelTypes;
  internalModel = null;
  externalModel = null;
  propertyChangeSupport.firePropertyChange ("modelTypes", oldModelTypes, modelTypes);
  fireMagPropsChange();
}

public int getInternalModelType() {
  return getModelTypes().getInternal();
}

public int getExternalModelType() {
  return getModelTypes().getExternal();
}



//----------- DATA ---------------



public double getKPIndex(double mjd) {
  if ( kpindex == null )
    return KPINDEX_DEFAULT;
  else{
    if ( mjd != kpindex.getLastMjd() ) {
      Double dd = (Double)kpindex.getValueAt(mjd,1);
      if ( dd == null ) KPINDEX_LAST = KPINDEX_DEFAULT;
      else KPINDEX_LAST = dd.doubleValue();
    }
    return KPINDEX_LAST;
  }
}

public double[] getIMF(double mjd) {
  if ( imf == null)
    return IMF_DEFAULT;
  else {
    Double dd;
    for ( int i=1; i<= 3; i++) {
      dd = (Double)imf.getValueAt(mjd,i);
      if ( dd == null ) {
          IMF_LAST = IMF_DEFAULT;
          break;
      }
      else IMF_LAST[i-1] = dd.doubleValue();
    }
    return IMF_LAST;
  }
}

public double getSWP(double mjd) {
  if ( swp == null)
    return SWP_DEFAULT;
  else {
    if ( mjd != swp.getLastMjd() ) {
      Double dd = (Double)swp.getValueAt(mjd,1);
      if ( dd == null ) SWP_LAST = SWP_DEFAULT;
      else SWP_LAST = dd.doubleValue();
    }
    return SWP_LAST;
  }
}

public double getDSTIndex(double mjd) {
  if ( dstindex == null)
    return DSTINDEX_DEFAULT;
  else {
    if ( mjd != dstindex.getLastMjd() ) {
      Double dd = (Double)dstindex.getValueAt(mjd,1);
      if ( dd == null ) DSTINDEX_LAST = DSTINDEX_DEFAULT;
      else DSTINDEX_LAST = dd.doubleValue();
    }
    return DSTINDEX_LAST;
  }
}

public double getMachNumber(double mjd) {
  if ( machnumber == null)
    return MACHNUMBER_DEFAULT;
  else {
    if ( mjd != machnumber.getLastMjd() ) {
      Double dd = (Double)machnumber.getValueAt(mjd,1);
      if ( dd == null ) MACHNUMBER_LAST = MACHNUMBER_DEFAULT;
      else MACHNUMBER_LAST = dd.doubleValue();
    }
    return MACHNUMBER_LAST;
  }
}

public double getSint(double mjd) {
  return getTrans(mjd).getSint();
}

public double getCost(double mjd) {
  return getTrans(mjd).getCost();
}

public double getDipoleTilt(double mjd) {
  return getTrans(mjd).getDipoleTilt();
}


/** Returns footprint altitude (km)
 * @depricated since 0.001 Use #getFootprintAltitude()
 */
public double getAlt() {
  return alt;
}

/** Returns footprint altitude (RE)
 * @returns footprint altitude (RE)
 */
public double getFootprintAltitude() {
  //System.out.println("getFootprintAlt is Broken.");
  return footprintAltitude;
}


public double getXlim() {
  return xlim;
}

public Descriptors getDescriptors() {
    if (descriptors == null) {
    // Add default property descriptor for visible property.
    // each visual object can be hidden or shown.
      try {

        descriptors = new Descriptors();
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("editorVisible", this);
        pd.setDisplayName("EditorVisibility");

        BasicPropertyEditor editor = new PropertiesEditor(pd);
        addPropertyChangeListener("editorVisible", editor);
        addPropertyChangeListener(editor);
        pd.setPropertyEditor(editor);
        descriptors.put(pd);

      } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
        System.exit(0);
      }
    }
    return descriptors;
  }

  /** Getter for property editorVisible.
   * @return Value of property editorVisible.
   */
  public boolean isEditorVisible() {
    return editorVisible;
  }
  /** Setter for property editorVisible.
   * @param editorVisible New value of property editorVisible.
   */
  public void setEditorVisible(boolean editorVisible) {
    java.awt.Point pp = core.XYZwin.getLocation();
    magPropsEditor.setLocation(pp.x + getCore().XYZwin.getSize().width, pp.y);
    magPropsEditor.setVisible(editorVisible);
    this.editorVisible = editorVisible;
  }

  public void fireMagPropsChange() {
    magPropsChangeSupport.fireMagPropsChange();
  }

}

// ---------------------------------------------------------

class MagPropsChangeSupport {

  private Vector listeners = new Vector();
  private Object source = null;


  /** Creates new MagPropssChangeSupport */
  public MagPropsChangeSupport(Object source) {
    this.source = source;
  }

  public void addMagPropsChangeListener (MagPropsChangeListener listener) {
    listeners.addElement(listener);
  }

  public void removeMagPropsChangeListener (MagPropsChangeListener listener) {
    listeners.removeElement(listener);
  }

  public void fireMagPropsChange(MagPropsEvent evt) {
    Enumeration e = listeners.elements();
    while (e.hasMoreElements())
      ((MagPropsChangeListener)(e.nextElement())).magPropsChanged(evt);
  }

  public void fireMagPropsChange() {
    MagPropsEvent evt = new MagPropsEvent(source);
    fireMagPropsChange(evt);
  }

  public boolean hasListener(MagPropsChangeListener listener) {
    return listeners.contains(listener);
  }

}
