/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/mag/Trace.java,v $
  Date:      $Date: 2000/06/08 14:21:33 $
  Version:   $Revision: 1.12 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 * Trace.java
 *
 * Created on March 24, 2000, 1:26 PM
 */
 
package ovt.mag;

import ovt.*;
import ovt.mag.*;
import ovt.mag.model.magnetopause.*;
import ovt.util.*;
import ovt.event.*;
import ovt.object.*;
import ovt.datatype.*;
import ovt.interfaces.*;


/** 
 *
 * @author  root
 * @version 
 */
public class Trace extends Object {
  
  // field-line tracing constants
public static final double  EPS	= 0.001;
public static final double  HMIN = 0.001;
public static final double  FF = 1.01;
public static final int  MAXCOUNT = 300;
public static final double  STMIN = 0.01;

//public static final double	STEPMAX	= 4.0;
public static final double	STEPMAX	= 1.0;

public static final int	MXX = 200; //max number of points in tracelin
public static final int    NMAX = 3;
public static final int    IMAX = 11;
public static final int    NUSE = 7;
public static final double STEPMIN = 0.002;
public static final double RMAX = 100.0;

  protected MagProps magProps;
  
  /** Creates new Trace */
  public Trace(MagProps magProps) {
    
  }
  
//------------------- field-line tracing  ---------------------------




/** 
 *    field-line tracing 
 * @param
 *    geo(3):   initial position (geo) units of re=6371.2 km 
 *    alt:      altitude of footprint (km) 
 *    where=1   trace to alt in the north 
 *         =0   trace to geomagnetic equator (alt not used)
 *         =-1  trace to alt in the south 
 * @return
 *    foot(3):  final vector re in geo coordinates
 */
/*
public double[] traceigrf(double[] geo, double alt, int where) {
    
    double  hmax, rmin, down, r, s = 0;
    int found = 0;
    double  hnext, rlast;
    int error = 0, stepcount = 0;
    double  rf, bv[] = new double[3], rv[] = new double[3];
    double  rvlast[] = new double[3];
    double  hin, dir;
	double[] foot = new double[3]; // - output
    int i;

	BsstepOutput bout;
	
    // Function Body 

    double gma[] = geo2gma(geo, 0);
    r = Vect.absv(geo);
	
    rlast = r; // Compiler said, that rlast may have not been initialized :(

    for (i=0; i<3; i++)
		rv[i] = geo[i];

    if (where != 0) {
        rmin = alt / Const.RE + 1.;
        dir = (double) where;
        down = 1.;
        if (rmin > r) {
            down = -1.;
            dir = -dir;
        }
        hnext = dir;
        while ((found == 0) && (stepcount < MAXCOUNT)) {
            for(i=0; i<3; i++) rvlast[i] = rv[i];
            rlast = r;
            hin = hnext;
            hmax = Math.abs(rlast - rmin) * FF;

            if (Math.abs(hin) > hmax)
                hin = hmax * dir;

            if (Math.abs(hin) < STMIN)
                hin = dir * STMIN;

            bout = bsstep(rv, 3, s, hin, EPS, HMIN, hnext, new rhanda(this));
            s = bout.x;
	    hnext = bout.hnext;
	    error = bout.error;
			
	    r = Vect.absv(rv);
            if ((r - rmin) * down < .01)
                found = 1;
            ++stepcount;
        }
        rf = (rmin - r) / (rlast - r);

        for (i=0; i<3; i++) foot[i] = rv[i] - (rv[i] - rvlast[i]) * rf;
			
    } else //  trace to equator     {
        bv = igrf(geo);
        r = Vect.dot(geo, bv);
        dir = Utils.new_sign(1.0, r);
        hnext = dir;
        while ((found == 0) && (stepcount < MAXCOUNT)) {
            hin = hnext;
            hmax = Math.abs(gma[2]) * FF;
            if (Math.abs(hin) > hmax)
                hin = dir * hmax;

            if (Math.abs(hin) < STMIN)
                hin = dir * STMIN;

            bout = bsstep(rv, 3, s, hin, EPS, HMIN, hnext, new rhanda(this));
			s = bout.x;
			hnext = bout.hnext;
			error = bout.error;
			
            gma = geo2gma(rv, 0);

            if (dir * gma[2] > -.01)
                found = 1;

            if (Vect.absv(rv) > 500.)
                found = 1;

            ++stepcount;
        }

        //for (d1 = foot, d2 = rv; d1 < foot + 3; ) *d1++ = *d2++;
		for (i=0; i<3; i++)
			foot[i] = rv[i];
    }

    if (stepcount > 299) {
        System.err.println("stepcount to large in traceigrf");
        System.exit(-1);
    }
	return foot;
}

*/


/******************************************************************
 FUNCTION:
  first closed field line starting from rv(3) in the direcion dir(3) 
  mjd,model,rv(3)  =  standard variables
  dir(3):  search direction from rv(3) 
  alt (km) altitude of required footprint
  foot(3) :  footprint position (re)
    idir= 1 north 
        =-1 south
  epst = tolerance (re) for determination of the origin of last closed rv(3)
******************************************************************/

public static void lastline (MagProps magProps, double mjd, double rv[], double dir[], int idir, double alt) {
	lastline(magProps, mjd, rv, dir, MagProps.xlim, alt, idir, Const.ERR_MP);
}

public static void lastline(MagProps magProps, double mjd, double rv[], double dir[], double xlim, 
				double alt, int idir, double epst) {
    // Local variables 

    double  ftol, rmin,  swap;
    MagPoint foot;
    int i;
    double  delta,rion, rfoot;
    int go;
    double  rv2[] = new double[3];
    int gout;
    double  gsr[] = new double[3];


    // Function Body 

    ftol = (epst < .005) ? .005 : epst;
    delta = 1.0;
    Vect.normf(dir, delta);
    rmin = alt / Const.RE + 1.2;
    if (Vect.absv(rv) < rmin) {
        System.err.println("lastline: incorrect alt or rv");
        System.exit (-1);
    }

    foot = foot_ns(magProps, mjd, rv, xlim, alt, idir);
    
    rfoot = Vect.absv(foot.gsm);
	
    gout = (rfoot < rmin) ? 1 : -1;

    go = 1;
    while (go > 0) {
		// We'v found a true footprint.
	for (int j=0; j<3; j++) {
	  rv2[j] = rv[j];
	  rv[j] += dir[j] * gout; 
	}
		
        foot = foot_ns(magProps, mjd, rv, xlim, alt, idir);
        //rfoot = Vect.absv(foot);
        rfoot = Vect.absv(foot.gsm);

	if ((Vect.absv(rv)>RMAX) || (Vect.absv(rv)<3.0)) go = 0;
        if (gout < 0) {
            if (rfoot < rmin)
                go = -1;
        } else {
            if (rfoot > rmin) {
                go = -1;
                for (int j=0; j<3; j++) {
		  swap  = rv[j];
		  rv[j] = rv2[j];
		  rv2[j] = swap;
	        }
            }
        }
    }
    if (go == 0) {
	System.err.println("lastline problem");
        return;  // no need to divide interval 
	}

	/*
	printf("last: rv=%.1lf %.1lf %.1lf foot=%.1lf %.1lf %.1lf\n",
	 rv[0],rv[1],rv[2],foot[0],foot[1],foot[2]);
	 */

    while (delta > ftol) {
        delta /= 2.;
        for (int j=0; j<3; j++)
	  gsr[j] = (rv[j] + rv2[j]) * 0.5;
			
        foot = foot_ns(magProps, mjd, gsr, xlim, alt, idir);
        
	rfoot = Vect.absv(foot.gsm);
        if (rfoot < rmin) {
            for (int j=0; j<3; j++) rv[j] = gsr[j];
				
	// tolerance proportional to gyroradius of 1 keV proton 
	
	double bv[] = magProps.bv(rv, mjd);
	
	rion = 4.58e3/Vect.absv(bv)/Const.RE;
	
	if(rion<0.02) rion=0.02;
	  ftol=rion;
        } else {
            for (int j=0; j<3; j++) rv2[j] = gsr[j];
        }
    }
    //return (1);
}



// was : returns -1 outside mag 0= not enough steps (mx) 1=ok  

public static Fieldline traceline(MagProps magProps, double mjd, double[] rv, double step, int is) {
	return traceline(magProps, mjd, rv, magProps.getAlt(), step, is, magProps.getXlim(), Const.NPF);
}

public static Fieldline traceline(MagProps magProps, double mjd, double[] rv, double step, int is, double alt) {
	return traceline(magProps, mjd, rv, magProps.getAlt(), step, is, magProps.getXlim(), Const.NPF);
}

/** returns fieldline
 * @param magModel Magnetic model to use for tracing
 * @param mjd time
 * @param rv initial position in gsm
 * @param alt altitude of footprint (if alt = 0 tracing is made to equatorial plane or xlim)
 * @param step <CODE>+</CODE> - parrallel to <B>b</B>
 * <CODE>-</CODE> - antiparrallel to <B>b</B>
 * @param is <CODE>0 - automatic </CODE>(optimal) step size
 * <CODE>1 - step size = step * sqrt(r)</CODE> (enforced)
 * @param xlim maximum distance in the tail
 * @param mx maximum number of points
 * @return Field Line
 */
public static Fieldline traceline(MagProps magProps, double mjd, double[] rv, 
                double alt, double step,int is, double xlim, int mx) {
  // Local variables 

  Fieldline f_l = new Fieldline(mjd);
  double  rrlast;
  double hmax; // adjustable step
  double rmin;
  int i, tocentre;
  double[] r = new double[3];
  double s, hnext;

  int error;
  double  rf, rr, dz, dsmax;
  double  dir, hin=0;

  double cosTheta, shueR;
  
  BsstepOutput bout;

  MagModel magModel = (MagModel)magProps;
  
  dir = Utils.new_sign(1.0, step);
  rrlast = Vect.absv(rv);
  dz=0.0;
  dsmax=1.0;

 
  if (alt > 1.00) {
    tocentre = 1;
    rmin = alt / Const.RE + 1.0;
  } else {
    tocentre = 0;
    rmin = 0.0;
  }

  s = 0.;
  r[0] = rv[0];
  r[1] = rv[1];
  r[2] = rv[2];
  // find the |B|

  f_l.add(new MagPoint( (double[])r.clone(), magModel.bv(r, mjd), mjd), s);

  int n = 0;

  hnext = Math.abs(step) * dir * Math.sqrt(rrlast);
  error = 0;

  for (i = 1; i < mx; i++) {
    if (is == 0)
    hin = hnext;
    else {
      hin = step * Math.sqrt(rrlast);
      if (Math.abs(hin) > Math.abs(hnext))
      hin = hnext;
    }

    hmax = Math.abs(rrlast - 1.01);
    
    if ((tocentre == 0) && (hmax > Math.abs(r[2])*1.3)) hmax = Math.abs(r[2])*1.3;
    if (hmax > STEPMAX) hmax = STEPMAX;

    if (Math.abs(hin) > hmax)
    hin = dir * hmax;
    if (Math.abs(hin) < STEPMIN)
    hin = dir * STEPMIN;

    bout = bsstep(r, 3, s, hin, EPS, HMIN, hnext, new rhandb(magModel, mjd));
    s = bout.x;
    hnext = bout.hnext;
    error = bout.error;
    
    rf = r[1] * r[1] + r[2] * r[2];
    rr = Math.sqrt(r[0] * r[0] + rf);

    
    f_l.add(new MagPoint( (double[])r.clone(), magModel.bv(r, mjd), mjd), s);
    //dz=r[0]-r_last[0]; ??

    n = i + 1;

    // check if Clipping is allowed
    if(ovt.mag.MagProps.isMPClipping()==true){  //added by kono
      // check if we are inside "Shue" magnetopause
      cosTheta = r[0]/rr;
      shueR = Shue97.getR(cosTheta, magProps.getSWP(mjd), magProps.getIMF(mjd)[2]);
      if (rr - shueR > 0.5) // we are outside
        return f_l;
    }
    
    if (tocentre == 0) {
      if (dir * r[2] > -STEPMIN) return f_l;
      
      //if (rf > RMAX*RMAX) return f_l;
      
      if (r[0] > 14.) return f_l;
      
      if (error != 0) return f_l;
      
      if (r[0] < xlim + STEPMIN) return f_l;
      if ((rr < rrlast) && (r[0] < 0.0)) return f_l;
      
    } else { 
      if (r[0] < xlim)
      error = 1;
      if (rf > RMAX * RMAX)
      error = 1;
      if (error == 1) {
        n -= 1;
        f_l.removeLastPoint();
        //System.err.println(".");
        return f_l; // was -1 here!!!. OutsideMsException should be thrown here
      }
      if (rr < rmin ) {
        rf = (rmin - rr) / (rrlast - rr);
        /*xx[n-1] -= (xx[n-1] - xx[n-2]) * rf;
        yy[n-1] -= (yy[n-1] - yy[n-2]) * rf;
        zz[n-1] -= (zz[n-1] - zz[n-2]) * rf;*/

        double[] a = (double[])(f_l.lastPoint().gsm).clone();
        double len = f_l.length();
        f_l.removeLastPoint();
        for (int k=0; k<3; k++) a[k] -= (a[k] - f_l.lastPoint().gsm[k]) * rf;
        f_l.add(new MagPoint(a, magModel.bv(a, mjd), mjd), len);

        //System.out.println("(rr < rmin )");
        return f_l;
      }
    }
    rrlast = rr;
  }
  System.err.println("traceline: too small mx="+i+"  init = "+ rv[0]+"\t"+rv[1]+"\t"+rv[2]);
  System.err.println("                   end  = " + r[0]+"\t"+r[1]+"\t"+r[2]+"\t hin="+hin );
  System.err.println("Not enouh steps!");

  return f_l; // was 0 here!!!. NotEnougStepsException should be thrown here
}




/** integration of ordinary differential equations
 *  bulirsch-stoer method
 *  see: numerical recipes , w. press, p 566. 1986.
 * @param y (ndim)  initial values at x
 * @param ndim     number of dependent variables (maximum 9)
 * @param x       independent variable (time)
 * @param htry     step size (can be quite big)
 * @param eps      maximum permissible error for y()
 * @param hmin     minimum step (signal with error=2)
 *  output:
 * @param y output:       updated at new x
 * @param x output:      new value  (=x + htry ! not necessarily)
 * @param hnext output:   recomended value for the next step
 * @param error output: .ne. 0 signals user defined error in SUBROUTINE right()
 *                 or hmin in bsstep()
 * 
 *  user has to supply a routine for right hand side derivatives:
 *  SUBROUTINE right (x,y,dydx,error)
 *  REAL*8 x,yy(10),dydx(10)
 */

// output[0] = x;
//		 [1] = hnext
//		 [2] = error;


protected static BsstepOutput bsstep(double[] y, int ndim, double x, double htry, 
			double eps, double hmin, double hnext, RightHand right) {
	
    /* Local variables */

    int  nseq[] = {
        2, 4, 6, 8, 12, 16, 24, 32, 48, 64, 96 }; // [IMAX=11]
	double  temp, xsav, yerr[] = new double[NMAX], xest, 
			ysav[] = new double[NMAX], yseq[] = new double[NMAX], h;
    int i, error;
    double  dysav[] = new double [NMAX], errmax;
    int pow2, tmpseq;

    /* Function Body */

    h = htry;
    xsav = x;
    for (i=0; i<ndim; i++) ysav[i] = y[i];
	
    // Chech if it is ok to integrate.
    error = right.rhand(x, ysav, dysav);
    
    if (error != 0) return new BsstepOutput(x, hnext, error);

    tmpseq = nseq[NUSE-2];
    while (Math.abs(h) > hmin) {
      for (i=0; i < 11; i++) {
		
            error = mmid(ysav, dysav, ndim, xsav, h, nseq[i], yseq, right);
            if (error != 0)
                return new BsstepOutput(x, hnext, error);
            temp = h / nseq[i];
            xest = temp * temp;
            rzextr(i, xest, yseq, y, yerr, ndim, NUSE);
            errmax = 0.;

            for (int j=0; j<ndim; j++)
	      if (errmax < Math.abs(yerr[j])) errmax = Math.abs(yerr[j]);
			
            if (errmax < eps) {
                x += h;
                temp = (double) tmpseq / nseq[i];
                if (temp > 2.)
                    temp = 2.;
                hnext = h * temp;

                // maximum factor 2 step increase 

                return new BsstepOutput(x, hnext, error);
            }
        }

        pow2 = 1;
		
		h = h * 0.25 / (double)(pow2*Math.pow(2, (IMAX - NUSE) / 2));
    }

    System.err.println("bsstep error=2: too large HMIN");
    // error = 2;
	return new BsstepOutput(x, hnext, error);
}


/**
 *
 *	y[NMAX], dxdy[NMAX], 
 *	ndim - dimention of y[], dxdy[]
 *	yout[NMAX]
 *	NMAX = 3  ;)
 */


protected static int mmid(double y[], double dydx[], int ndim, double xs, double htot, 
			int nstep, double yout[], RightHand right) {

    /* Local variables */

    double  swap, h;
    int n;
    double  x, h2, ym[] = new double[10], yn[] = new double[10];
	int error =0;
    /* Function Body */

    h = htot / nstep;

    /*for ( d1 = ym, d2 = yn, d3 = y, d4 = dydx; d1 < ym + ndim; ) {
        *d1++ = *d3;
        *d2++ = *d3++ + h * *d4++;
    }*/

	for (int i=0; i<ndim; i++) {
		ym[i] = y[i];
		yn[i] = y[i] + h*dydx[i];
	}
	

    x = xs + h;
    error = right.rhand(x, yn, yout);
    if (error != 0)
        return error;
    h2 = h * 2.;
    for (n = 2; n <= nstep; ++n) {
      for (int i=0; i<ndim; i++) {
			swap = ym[i] + h2 * yout[i];
			ym[i] = yn[i];
			yn[i] = swap;
      }
		
        x += h;
        error = right.rhand(x, yn, yout);
        if (error != 0)
            return error;
    }

    for (int i=0; i<ndim; i++)
      yout[i] = (ym[i] + yn[i] + h*yout[i]) * 0.5;

    return error;
}


/** yest[NMAX], yz[NMAX], dy[NMAX]
 * ndim - dimention of yest[NMAX], yz[NMAX], dy[NMAX]
 * NMAX = 3 ;)
 */


protected static void rzextr(int iest, double xest, double yest[], double yz[], 
				double dy[], int nv, int nuse) {

   // Local variables 
    double[] d = new double[70], x = new double[11];
    double  b, c;
    double  v, b1;
    int m1;
    double  fx[] = new double[7], yy, ddy=0;

    // Function Body
    x[iest] = xest;
    if (iest == 0) {
		for (int i=0; i<nv; i++) {
			yz[i]  = yest[i];
			d[i*7] = yest[i];
			dy[i]  = yest[i];
		}
    } else {
        //--nuse;
        m1 = Math.min(iest, nuse);

		for (int k=0; k<m1-1; k++) 
			fx[k + 1] = x[iest - k] / xest;
		
		for (int j=0; j<nv; j++) {
			yy = yest[j];
			v = d[j*7];
			c = yy;
			d[j*7] = yy;
			for (int k = 1; k<m1; k++) {
				b1 = fx[k] * v;
				b = b1 - c;
				if (Math.abs(b) > 1e-22) {
					b = (c - v) / b;
					ddy = c * b;
					c = b1 * b;
				} else
					ddy = v;
				if (k != m1-1) v = d[j*7 + k];
				
				d[j*7 + k] = ddy;
				yy += ddy;
			}
			dy[j] = ddy;
			yz[j] = yy;
		}
    }
}



/*
public MagPoint foot_ns(double[] rv, int idir) {
	return foot_ns(rv, MagProps.xlim, MagProps.alt, idir) ;
}*/

/** 
 * Returns magnetic footprint in the north (idir=1) or south (idir=-1) 
 * @param rv(3) initial position (re),
 *  alt = altitude (km), north (idir=1) or south (idir=-1)
 *  @return foot print (3) in (re) 
 */


public static MagPoint foot_ns(MagProps magProps, double mjd, double[] rv, double xlim, double alt, int idir) {

    // Local variables 

    double   halt, step; //static 
    int  n=0;
    MagPoint foot; // - Output
	
    // Function Body 

    if (Vect.absv(rv) < 1) {
        System.err.println("foot_ns: Input vector below the surf");
        System.exit (-1);
    }

	step = (idir < 0) ? -0.1 : 0.1;

	Fieldline f_l = traceline(magProps, mjd, rv, alt, step, 0, xlim, MXX);
	
	/*System.out.println("fp: ["+f_l.size()+"] "+
		f_l.lastPoint()[0]+"\t"+f_l.lastPoint()[1]+"\t"+f_l.lastPoint()[2]);

	for (int i=0; i<f_l.size(); i++)
		System.out.println("fp: ->"+i+" "+
			f_l.point(i)[0]+"\t"+f_l.point(i)[1]+"\t"+f_l.point(i)[2]);*/

    if (f_l.size() > 0) 
        return f_l.lastPoint();
    else {
      double[] gsm = new double[]{ 999.0, 999.0, 999 }; 
      double[] bv = new double[]{ 0, 0, 0 }; 
      System.err.println("foot_ns: wrong traceline");
      return new MagPoint(gsm, bv, mjd);
    }
}
 }



//-----------------------------------------------------------------------




/* right hand side for field line tracing with bsstep */

interface RightHand {

// WHY DO WE NEED double s ?????

public int rhand(double s, double r[], double drds[]);

}




class rhanda implements RightHand {
  protected MagModel magModel;
  protected double mjd;
rhanda(MagModel magModel, double mjd) { 
  this.magModel = magModel;
  this.mjd = mjd;
}
public int rhand(double s, double r[], double drds[]) {
    double  r2, rv[] = new double[3];
    int i, error = 0;

    /* right hand side for field line tracing with bsstep */

    for (i=0; i<3; i++) rv[i] = r[i];
    
    //double bv[] = magModel.igrf(rv); !!!!!!
    double bv[] = magModel.bv(rv, mjd);
    double b = Vect.absv(bv);
    for (i=0; i<3; i++) drds[i] = bv[i] / b;
    return error;
}

}
	

/*  contribution of the ring current to EXTERNAL field (nt) */
/*  tsyganenko long                                         */

class rhandb implements RightHand {
  protected MagModel magModel;
  protected double mjd;
rhandb(MagModel magModel, double mjd) { 
  this.magModel = magModel;
  this.mjd = mjd;
}
public int rhand(double s, double r[], double drds[]) {
    double  r2, rv[] = new double[3];
    int i, error=0;
    for (i=0; i<3; i++) rv[i] = r[i];

    double bv[] = magModel.bv(rv, mjd);
    double b = Vect.absv(bv);
    for (i=0; i<3; i++) drds[i] = bv[i] / b;

    /* return error code if outside the magnetosphere */

	// before magnetopause
    if ((rv[0] > 13) || (rv[0] < -52)) error = 1;

    r2 = rv[1] * rv[1] + rv[2] * rv[2];
    if (r2 > Trace.RMAX * Trace.RMAX) error = 1;
    return error;
}

}

// output[0] = x;
//		 [1] = hnext
//		 [2] = error;


class BsstepOutput {

	public double x=0;
	public double hnext = 0;
	public int error=0;

BsstepOutput(double ex, double hanext, int err) {
	x = ex;
	hnext = hanext;
	error = err;
}
}

