/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/BowShock.java,v $
  Date:      $Date: 2000/06/13 13:30:31 $
  Version:   $Revision: 1.12 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object;

import ovt.*;
import ovt.mag.*;
import ovt.mag.model.bowshock.*;
//import ovt.calc.*;
import ovt.event.*;
import ovt.beans.*;
import ovt.object.editor.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import vtk.*;

import java.lang.Math;
import java.util.*;
import java.beans.*;
import java.awt.event.*;

import javax.swing.*;

public class BowShock extends VisualObject implements 
  TimeChangeListener, CoordinateSystemChangeListener, MagPropsChangeListener {

  public static final int SURFACE = 0;
  public static final int WIREFRAME = 1;
  
  protected vtkActor actor;
  

/** Holds value of property representation. */
private int representation;

public BowShockPropertyEditor bowShockPropertyEditor = null;

/** Holds value of property customizerVisible. */
private boolean customizerVisible = false;

public BowShock(OVTCore core) { 
  super(core, "Bow Shock", core.getImagesDir()+"bowshock.gif");
  bowShockPropertyEditor = new BowShockPropertyEditor(core.getMagProps());
  Descriptors descriptors = getDescriptors();
  try {
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("customizerVisible", this);
        pd.setDisplayName("customizerVisible");
        
        BasicPropertyEditor editor = new BooleanEditor(pd, MenuPropertyEditor.SWITCH);
        editor.setTags(new String[]{"Properties...", "Hide properties"});
        addPropertyChangeListener("customizerVisible", editor);
        pd.setPropertyEditor(editor);
        descriptors.put(pd);
        
  } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
  }
}


public void show() {
  actor = getActor();
  getRenderer().AddActor(actor);
  if (representation == SURFACE) setRepresentationToSurface();
  else setRepresentationToWireframe();
  rotate();
}

public void hide() {
  getRenderer().RemoveActor(actor);
}


public void setVisible(boolean visible) {
    if (visible != isVisible()) {
      if (visible) show();
      else hide();
      super.setVisible(visible);
    }
}

protected void setRepresentationToWireframe() 
	{ actor.GetProperty().SetRepresentationToWireframe(); }

protected void setRepresentationToSurface() 
	{ actor.GetProperty().SetRepresentationToSurface(); }


protected vtkActor getActor() {
  //System.out.println("getActor (BowShock)");
  if (!isValid()) update();
    if (actor == null) {
	// create actor
	// Here we go!

	double[] gsm = new double[3];
	double[] rv = new double[3];
	
	double thetaMax = 140*Const.D_TO_R;
	
	double phi, theta, dPhi, dTheta, r;
        int phiResolution = 50, thetaResolution = 60;
	double x, y, z;
	
	dPhi   = 2.*Math.PI/(phiResolution );
	dTheta = thetaMax/(thetaResolution );
	
	vtkPoints points = new vtkPoints();
        
        double swp = getMagProps().getSWP(getMjd());
        
        double machNumber = getMagProps().getMachNumber(getMjd());
        //System.out.println("MachNumber = "+machNumber);
        double cosTheta, sinTheta, cosPhi, sinPhi;
        
	int i, j;
        int sizex = phiResolution + 1;
        int sizey = thetaResolution + 1;
        
	for (theta=0, i=0; i<sizey; theta+=dTheta, i++) {
          sinTheta = Math.sin(theta);
          cosTheta = Math.cos(theta);
	  for (phi=0, j=0; j<sizex; phi+=dPhi, j++) {
            sinPhi = Math.sin(phi);
            cosPhi = Math.cos(phi);
            r = Bowshock99Model.getR(cosTheta, swp, machNumber);
            x = r * cosTheta;
            z = r * sinTheta * cosPhi;
            y = r * sinTheta * sinPhi;
            points.InsertNextPoint(x, y, z);
	  }
	}
        
	vtkStructuredGrid sgrid = new vtkStructuredGrid();
			sgrid.SetDimensions(sizex, sizey,1);
			sgrid.SetPoints(points);
		
	vtkStructuredGridGeometryFilter gfilter = new vtkStructuredGridGeometryFilter();
			gfilter.SetInput(sgrid);
			gfilter.SetExtent(0,sizex,0,sizey,0,0);
                        

	vtkPolyDataMapper mapper = new vtkPolyDataMapper();
			mapper.SetInput(gfilter.GetOutput());
		
        actor = new vtkActor();
        	actor.SetMapper(mapper);
        	actor.GetProperty().SetColor(0, 1, 0);
                actor.GetProperty().SetOpacity(0.1);		
  }
  return actor;
}

public void rotate() {
    Matrix3x3 m3x3 = getTrans(getMjd()).gsm_trans_matrix(getCS());
    actor.SetUserMatrix(m3x3.getVTKMatrix()); 
}



public void update() {
  //System.out.println("Update...");
  actor = null;
  valid = true;
}

/** Getter for property representation.
 * @return Value of property representation.
 */
public int getRepresentation() {
  return representation;
}
/** Setter for property representation.
 * @param representation New value of property representation.
 */
public void setRepresentation(int representation) {
  this.representation = representation;
}
public void timeChanged(TimeEvent evt) {
  if (isVisible()) rotate();
}
public void coordinateSystemChanged(CoordinateSystemEvent evt) {
  if (isVisible()) rotate();
}
public void magPropsChanged(MagPropsEvent evt) {
  invalidate();
  if (isVisible()) {
    hide();
    show();
  }
}

/** Getter for property customizerVisible.
 * @return Value of property customizerVisible.
 */
public boolean isCustomizerVisible() {
  return customizerVisible;
}
/** Setter for property customizerVisible.
 * @param customizerVisible New value of property customizerVisible.
 *
 * @throws PropertyVetoException
 */
public void setCustomizerVisible(boolean customizerVisible) throws java.beans.PropertyVetoException {
  boolean oldCustomizerVisible = this.customizerVisible;
  vetoableChangeSupport.fireVetoableChange("customizerVisible", new Boolean (oldCustomizerVisible), new Boolean (customizerVisible));
  this.customizerVisible = customizerVisible;
  bowShockPropertyEditor.setVisible(customizerVisible);
  propertyChangeSupport.firePropertyChange ("customizerVisible", new Boolean (oldCustomizerVisible), new Boolean (customizerVisible));
}
}
