/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/Dumper.java,v $
  Date:      $Date: 2000/06/14 13:50:04 $
  Version:   $Revision: 1.8 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object;

import ovt.util.*;
import ovt.object.*;
import ovt.datatype.*;

import java.io.*;
import java.lang.*;
import java.util.*;

/** Writer of OrbitMonitorModule's
 * @author kono
 */
public class Dumper {
  public static final String TAB = "\t";
  public static final String EXT = ".dump";
  
  private String fileName = "";
  private BufferedWriter writer;
  
  private boolean POSITION = true;    // Default values
  private boolean FOOT_PRINTS = true;
  private boolean MP_DIST = false;
  private boolean SPIN = false;
  
  // true - One header at the beginning of file (Cartoon mode)
  // false - adding header in every record
  private boolean HEADER = true;
  
  /**
   * -1 - output not initialized or IO error
   * 0 - output file is opened
   * >0 - number of writed records to output file
   */
  private int status = -1;

  public Dumper(){
  }
  
  public Dumper(String fn){
    initialize(fn);
  }
  
  public Dumper(String fn,boolean pos,boolean fp,boolean mp,boolean spin){
    initialize(fn);
    this.setupDump(pos,fp,mp,spin);
  }
  
  public void initialize(String filename){
    this.fileName = new String(filename);
    try {
      FileWriter fw = new FileWriter(this.fileName, false);
      writer = new BufferedWriter(fw);
      status = 0;
    } catch (IOException e){
      System.err.println("IO error with file "+fileName+" : "+e.toString());
      status = -1;
    }
  }
  
  public void setupDump(String fn,boolean pos, boolean fp, boolean mp, boolean spin, boolean mode){
    this.HEADER = mode;
    setupDump(fn,pos,fp,mp,spin);
  }
  
  public void setupDump(String fn,boolean pos, boolean fp, boolean mp, boolean spin){
    try {
      if(getStatus()!=-1)
        this.writer.close();
    } catch (IOException e){
    }
    
    initialize(fn);
    setupDump(pos,fp,mp,spin);
  }
  
  public void setupDump(boolean pos, boolean fp, boolean mp, boolean spin){
    this.POSITION = pos;
    this.FOOT_PRINTS = fp;
    this.MP_DIST = mp;
    this.SPIN = spin;
  }
  
  public void writeDumpRecord(DumpRecord dumpRecord){
    if(dumpRecord==null || getStatus()==-1)
      return;
    try {
      String headStr = new String(TAB);
      String posStrLabel = dumpRecord.getField(DumpRecord.POS_LABEL);
      String fpStrLabel = dumpRecord.getField(DumpRecord.FOOT_LABEL);
      if(getStatus()==0){
        //Writing header
        if(POSITION)     //Writing current CS for S/C position
          writer.write(posStrLabel + ", ");
        if(FOOT_PRINTS)  //Writing current CS for footprints
          writer.write(fpStrLabel);
        writer.newLine();   // Adding NL symbol
      }// else
      if(!this.HEADER){
        // We have to save in every record: ... GEO SMC ...
        int beg,end;
        if(POSITION){
          beg = posStrLabel.indexOf('[')+1;
          end = posStrLabel.indexOf(']');
          headStr = headStr.concat(posStrLabel.substring(beg,end)+TAB);
        }
        if(FOOT_PRINTS){
          beg = posStrLabel.indexOf('[')+1;
          end = posStrLabel.indexOf(']');
          headStr = headStr.concat(posStrLabel.substring(beg,end)+TAB);
        }
      }
      
      // Dumping Time
      String str = dumpRecord.getField(DumpRecord.TIME) + headStr;
      
      if(POSITION){
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.POSX)));
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.POSY)));
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.POSZ)));
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.VEL)));
      }
      
      if(FOOT_PRINTS){
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.LAT1)));
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.LON1)));
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.LAT2)));
        str=str.concat(digitalFilter(dumpRecord.getField(DumpRecord.LON2)));
      }
      
      writer.write(str);  // Writing data as one long string
      writer.newLine();   // Adding NL symbol
      
      ++status;
    } catch (IOException e){
      status = -1;
    }
  }
  
  public void close(){
    try {
      if(getStatus()!=-1)
        this.writer.close();
    } catch (IOException e){
    }
    status = -1;
  }
  
  public int getStatus(){
    return this.status;
  }
  
  // returns digital values
  public static String digitalFilter(String inpstr){
    final String empty = "0.0" + TAB;
    String outstr = "";
    boolean isAnyDigit = false;

    if(inpstr==null)
      return empty;
    for(int i=0;i<inpstr.length();++i){
      char c = inpstr.charAt(i);
      if(Character.isDigit(c) || c=='-' || c=='.' || Character.isSpaceChar(c))
        outstr=outstr.concat(new Character(c).toString());
      if(Character.isDigit(c))
        isAnyDigit = true;
    }
    outstr.replace(' ','\t');
    if(outstr.length()==0 || isAnyDigit==false)
      return empty;
    else
      return outstr + TAB;
  }

}
