/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/Earth.java,v $
  Date:      $Date: 2000/06/13 19:11:16 $
  Version:   $Revision: 1.20 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 * Earth.java
 *
 * Created on March 22, 2000, 8:22 AM
 */
 
package ovt.object;

import ovt.*;
import ovt.mag.*;
import ovt.util.*;
import ovt.event.*;
import ovt.beans.*;
import ovt.datatype.*;
import ovt.interfaces.*;
import ovt.object.editor.*;
import ovt.object.vtk.ProgrammableEarthSource;

import vtk.*;

import java.io.*;
import java.lang.*;
import java.beans.*;
import javax.swing.*;


/** 
 *
 * @author  root
 * @version 
 */
public class Earth extends VisualObject implements TimeChangeListener, 
                      CoordinateSystemChangeListener {

  public static final int CONTINENTS = 0;
  public static final int GRID       = 1;
  public static final int BOTH       = 2; //added by kono
  
  protected vtkAssembly[] actors = new vtkAssembly[3];
  protected vtkAssembly actor = null;
  /** Holds value of property earthType. */
  protected int earthType = GRID;
  /** The type of the earth to be used in the case of no conflicts */
  protected int prefferedEarthType = CONTINENTS;
  
  protected EarthTypeEditor earthTypeEditor;
  
  private OVTCore corex;
  
  /** Creates new Earth */
  public Earth(OVTCore core) {
    super(core, "The Earth", OVTCore.getImagesDir() + "globe.gif");
    this.corex = core;
    actors[CONTINENTS] = getContinentsActor();
    actors[GRID] = getGridActor();
    actors[BOTH] = getGridedContinentsActor();
    
    Descriptors descriptors = super.getDescriptors();
    
    try {
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("earthType", this);
        pd.setDisplayName("EarthType");
        
        earthTypeEditor = new EarthTypeEditor(pd);
        addPropertyChangeListener(earthTypeEditor);
        pd.setPropertyEditor(earthTypeEditor);
        descriptors.put(pd);
    } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
        System.exit(0);
    }
    try { setEarthType(CONTINENTS); } catch (PropertyVetoException e2) {}
    setVisible(true);
  }
  
  public vtkAssembly getContinentsActor() {
      // create actor  
      // create sphere geometry
  	vtkTexturedSphereSource tss = new vtkTexturedSphereSource();
  	tss.SetRadius(1.0);
  	tss.SetThetaResolution(40);
  	tss.SetPhiResolution(40);

      // map to graphics library
  	vtkPolyDataMapper earthMapper = new vtkPolyDataMapper();
  	earthMapper.SetInput(tss.GetOutput());

    // actor coordinates geometry, properties, transformation
  	vtkActor act = new vtkActor();
  	act.SetMapper(earthMapper);

    // load in the texture map
	vtkTexture atext = new vtkTexture();
	vtkPNMReader pnmReader = new vtkPNMReader();
	pnmReader.SetFileName(getCore().getImagesDir()+"earth.ppm");
	atext.SetInput(pnmReader.GetOutput());
	atext.InterpolateOn();
	act.SetTexture(atext);
        
    vtkAssembly assembly = new vtkAssembly();
    assembly.AddPart(act);
    return assembly;
  }
  
  public vtkAssembly getGridActor() {
    ProgrammableEarthSource earthSource = new ProgrammableEarthSource();
    earthSource.setRadius(1);
    earthSource.setPhiResolution(5);
    earthSource.setThetaResolution(5);
      
    vtkPolyData polyData = earthSource.GetOutput();
    
    vtkThresholdPoints thresholdPoints = new vtkThresholdPoints();
      thresholdPoints.SetInput(polyData);  
      thresholdPoints.ThresholdBetween(1, 1);
 
    // create slid surface
      
    vtkPolyDataMapper mapper = new vtkPolyDataMapper();
      mapper.SetInput(polyData);
      mapper.ScalarVisibilityOff();
      
    vtkActor actor1 = new vtkActor();
    actor1.SetMapper(mapper);
    actor1.GetProperty().SetColor(1, 1, 1);
    actor1.SetScale(0.999);
    
    // Grid 
    
    mapper = new vtkPolyDataMapper();
      mapper.SetInput(thresholdPoints.GetOutput());
      
    vtkActor act = new vtkActor();
    act.SetMapper(mapper);
    act.GetProperty().SetColor(1, 1, 1);
    
    vtkAssembly assembly = new vtkAssembly();
    assembly.AddPart(act);
    assembly.AddPart(actor1);
    
    return assembly;
  }

  //added by kono
  public vtkAssembly getGridedContinentsActor() {
        // create actor  
        // create sphere geometry
    vtkTexturedSphereSource tss = new vtkTexturedSphereSource();
    tss.SetRadius(1.0);
    tss.SetThetaResolution(80);
    tss.SetPhiResolution(80);

        // map to graphics library
    vtkPolyDataMapper earthMapper = new vtkPolyDataMapper();
    earthMapper.SetInput(tss.GetOutput());

        // actor coordinates geometry, properties, transformation
    vtkActor act = new vtkActor();
    act.SetMapper(earthMapper);

        // load in the texture map
    vtkTexture atext = new vtkTexture();
    vtkPNMReader pnmReader = new vtkPNMReader();
    pnmReader.SetFileName(getCore().getImagesDir()+"earth_grid.pnm");
    atext.SetInput(pnmReader.GetOutput());
    atext.InterpolateOn();
    act.SetTexture(atext);

    vtkAssembly assembly = new vtkAssembly();
    assembly.AddPart(act);
    
    ProgrammableEarthSource earthSource = new ProgrammableEarthSource();
    earthSource.setRadius(1);
    earthSource.setPhiResolution(5);
    earthSource.setThetaResolution(5);
      
    vtkPolyData polyData = earthSource.GetOutput();
    
    vtkThresholdPoints thresholdPoints = new vtkThresholdPoints();
      thresholdPoints.SetInput(polyData);  
      thresholdPoints.ThresholdBetween(1, 1);
 
    // create slid surface
      
    vtkPolyDataMapper mapper = new vtkPolyDataMapper();
      mapper.SetInput(polyData);
      mapper.ScalarVisibilityOff();
      
    vtkActor actor1 = new vtkActor();
    actor1.SetMapper(mapper);
    actor1.GetProperty().SetColor(1, 1, 1);
    actor1.SetScale(0.999);
    
    // Grid 
    
    mapper = new vtkPolyDataMapper();
      mapper.SetInput(thresholdPoints.GetOutput());
      
    vtkActor act2 = new vtkActor();
    act2.SetMapper(mapper);
    act2.GetProperty().SetColor(1, 1, 1);
    
    assembly.AddPart(act2);
    
    return assembly;
  }

  protected void show() {
    actor = actors[getEarthType()];
    getRenderer().AddActor(actor);
    rotate();
  }
  
  protected void hide() {
    if (actor != null) {
      getRenderer().RemoveActor(actor);
      actor = null;
    }
  }
  
  public void setVisible(boolean visible) {
    if (visible != isVisible()) {
      if (visible) show();
      else hide();
      super.setVisible(visible);
    }
  }
  
  public void rotate() {
    Matrix3x3 m3x3 = getTrans(getMjd()).trans_matrix(getPolarCS(), getCS());
    actor.SetUserMatrix(m3x3.getVTKMatrix()); 
  }
  
  public void timeChanged(TimeEvent evt) {
    if (isVisible()) rotate(); 
  }
  public void coordinateSystemChanged(CoordinateSystemEvent evt) {
    
    if (isVisible()) rotate();
    
    if (evt.getWindow() == Const.POLAR) {
      // it is not possible to show earth with continents 
      // if CS is not GEO
      if (evt.getNewCS() == CoordinateSystem.GEO) {
        earthTypeEditor.setEnabled(CONTINENTS, true);
        earthTypeEditor.setEnabled(BOTH, true);
        try { 
          setEarthType(prefferedEarthType);
        } catch (PropertyVetoException e2) {}
      } else { // new CS != GEO
        earthTypeEditor.setEnabled(CONTINENTS, false);
        earthTypeEditor.setEnabled(BOTH, false);
        // if the privious CS was GEO - store earthType.
        if (evt.getOldCS() == CoordinateSystem.GEO) {
          prefferedEarthType = getEarthType();
        }
        if (getEarthType() == CONTINENTS || getEarthType() == BOTH)
          try { 
            setEarthType(GRID);
          } catch (PropertyVetoException e2) {}
      }
    }
  }
  
  /** Getter for property earthType.
   * @return Value of property earthType.
   */
  public int getEarthType() {
    return earthType;
  }
  /** Setter for property earthType.
   * @param earthType New value of property earthType.
   *
   * @throws PropertyVetoException
   */
  public void setEarthType(int earthType) throws PropertyVetoException {
    int oldEarthType = this.earthType;
    if ((getPolarCS() != CoordinateSystem.GEO) && (earthType == CONTINENTS))
        throw new PropertyVetoException("Continents cannot be displayed in the " +
            CoordinateSystem.getCoordSystem(getPolarCS()) + " coordinate system",
            new PropertyChangeEvent(this, "earthType", new Integer(oldEarthType), new Integer(earthType)));
    
    vetoableChangeSupport.fireVetoableChange("earthType", new Integer (oldEarthType), new Integer (earthType));
    this.earthType = earthType;
    if (isVisible()) { hide(); show(); }
    propertyChangeSupport.firePropertyChange ("earthType", new Integer (oldEarthType), new Integer (earthType));
  }
  
  
  public static String getEarthType(int type) {
    switch (type) {
      case CONTINENTS:  return "Continents";
      case GRID      :  return "Grid";
      case BOTH      :  return "Continents & Grid";
    }
    throw new IllegalArgumentException("invalid earth type '"+type+"'");
  }

}
