/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/FieldlineModule.java,v $
  Date:      $Date: 2000/04/11 17:04:52 $
  Version:   $Revision: 1.12 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object;

import ovt.*;
import ovt.util.*;
import ovt.mag.*;
import ovt.event.*;
import ovt.datatype.*;
import ovt.interfaces.*;


import vtk.*;

import javax.swing.*;
import java.io.File;
import java.util.*;



/**
 * This is the fieldline tracing module for satellite, that should be added
 * to { @link ovt.datatypes.Sat }. 
 * @author Mykola Khotyaintsev
 * @see ovt.datatypes.AbstractSatModule
 */
public class FieldlineModule extends AbstractVisualSatModule
  implements MagPropsChangeListener {

  public static final int FL_2_EARTH   =  0;
  public static final int FL_2_EQUATOR  =  1;
  
  protected int fl_type = FL_2_EARTH;

  protected vtkActor actor = null;
  
  private Hashtable actors = new Hashtable();
  private double actorsMjd;


/** Create a FieldLinesSatModule */

public FieldlineModule(MainFieldlineModule mainModule, int fl_type) { 
	super(mainModule.getSat());
        setIcon(new ImageIcon(OVTCore.getImagesDir()+"fieldline.gif"));
	this.fl_type = fl_type;
	String name = "";
	if (fl_type == FL_2_EARTH)   name = "to Earth";
	if (fl_type == FL_2_EQUATOR) name = "to Equator";
	setName(name);
        setParent(mainModule);
  }

  
  
  /** Returns the field line type.
   * @return Field line type. {@see ovt.object.Sat}
   */
  public int getFLType() {
    return fl_type;
  }

  protected Fieldline getFieldline(int type, double mjd) {
    return getSat().getFieldline(type, mjd);
  }
  
  protected vtkActor getActor() {
    return getActor(getMjd());
  }
  
  private vtkActor getActor(double mjd) {
    if (!isValid()) validate();
    // look in hashtable
    vtkActor act = (vtkActor)actors.get(new Double(mjd));
    if ( act == null)  { // nothing found in hashtable
      Fieldline fl = getFieldline(getFLType(), getMjd());
      if (fl == null) {
        //System.out.println("fl = null");
        return null;
      }
      //System.out.println(" size = "+fl.size());
      act = ActorUtils.getActor(fl);
    }
    return act;
  }
	
  public void show() {
    actor = getActor();
    if (actor != null) {
      updateToCurrentCS();
      getRenderer().AddActor(actor);
    }
  }

  public void hide() {
    if (actor != null) { 
      getRenderer().RemoveActor(actor);
      actor = null;
    }
  }

  public void updateToCurrentCS() {
    if (actor != null) {
      Matrix3x3 trm = getTrans(getMjd()).gsm_trans_matrix(getCS());
      actor.SetUserMatrix(trm.getVTKMatrix());
    }
  }

  public void validate() {
    actors.clear();
    valid = true;
  }
	
  public void timeChanged(TimeEvent evt) {
    if (evt.getPropertyName().equals("time")) {
      invalidate();
    }
    
    if (isVisible()) {
      hide();
      show();
    }
  }
  
  public void setVisible(boolean visible) {
    if (isVisible() != visible) {
      if (visible) show();
      else hide();
      super.setVisible(visible);
    }
  }

  public void coordinateSystemChanged(CoordinateSystemEvent evt) {
    //System.out.println("FieldLineModel. CS chamged.");
    //invalidate();
    if (isVisible()) updateToCurrentCS();
    /*
    if (isVisible()) {
      hide();
      show();
    }*/
  }


  public void magPropsChanged(MagPropsEvent evt) {
    invalidate();
    if (isVisible()) {
      hide();
      show();
    }
  }
}
