/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/FrameModule.java,v $
  Date:      $Date: 2000/04/11 15:42:26 $
  Version:   $Revision: 1.3 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 * FrameModule.java
 *
 * Created on March 22, 2000, 10:22 AM
 */
 
package ovt.object;

import ovt.*;
import ovt.mag.*;
import ovt.event.*;
import ovt.beans.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import vtk.*;

import java.beans.*;

/** 
 *
 * @author  root
 * @version 
 */
public class FrameModule extends VisualObject {

  public static final int XOYPLANE = 1;
  public static final int XOZPLANE = 2;
  public static final int YOZPLANE = 3;

  protected vtkActor actor = null;
  
  private Frames parent;
  
  protected int type = XOYPLANE;
  /** frame paramiter */
  int l=5;
  /** frame paramiter */
  double dx = 5;
  
  /** Creates new FrameModule */
  public FrameModule(Frames frames, int type) {
    super(frames.getCore(), getName(type), frames.getCore().getImagesDir()+"frame.gif");
    parent = frames;
    setType(type);
  }
  
  public static String getName(int type) {
    switch (type) {
      case XOYPLANE : return "XOY";
      case XOZPLANE : return "YOZ";
      case YOZPLANE : return "XOZ";
      default: return "no name";
    }
  }
  
  protected void setType(int type) {
    if (type == XOYPLANE || type == XOZPLANE || type == YOZPLANE) {
      this.type = type;
    } else throw new IllegalArgumentException("type is not XOYPLANE || XOZPLANE || YOZPLANE");
  }
  
  public int getType() {
    return type;
  }
  
  protected vtkActor getActor() {
    if (actor == null) {
        
  	int i;
	vtkFloatScalars xCoords = new vtkFloatScalars();
	
	for (i=0; i<2*l+1; i++) xCoords.InsertScalar(i,(i-l)*dx); 
	
	vtkFloatScalars yCoords = new vtkFloatScalars();
	for (i=0; i<2*l+1; i++) yCoords.InsertScalar(i,(i-l)*dx); 
	
	vtkFloatScalars zCoords = new vtkFloatScalars();
	for (i=0; i<2*l+1; i++) zCoords.InsertScalar(i,(i-l)*dx); 		
	
	vtkRectilinearGrid rgrid = new vtkRectilinearGrid();
	
	rgrid.SetDimensions(2*l+1,2*l+1,2*l+1);
	rgrid.SetXCoordinates(xCoords);
	rgrid.SetYCoordinates(yCoords);
	rgrid.SetZCoordinates(zCoords);
	
        vtkRectilinearGridGeometryFilter plane = new vtkRectilinearGridGeometryFilter();
        plane.SetInput(rgrid);
        
        switch (getType()) {
          case XOYPLANE : plane.SetExtent(0,2*l,0,2*l,l,l); break;
          case XOZPLANE : plane.SetExtent(0,0,0,2*l,0,2*l); break;
          case YOZPLANE : plane.SetExtent(0,2*l,l,l,0,2*l); break;
        }
        
	vtkPolyDataMapper gridMapper = new vtkPolyDataMapper();
	gridMapper.SetInput(plane.GetOutput());
	
	actor = new vtkActor();
	actor.SetMapper(gridMapper);
	actor.GetProperty().SetRepresentationToWireframe();
	actor.GetProperty().SetColor(1,1,1);
	//actor.GetProperty().SetOpacity(0.5);
    }
    return actor;
  }

  protected void show() {
    getRenderer().AddActor(getActor());
  }
  
  protected void hide() {
    getRenderer().RemoveActor(getActor());
  }
  
  public void setVisible(boolean visible) {
    if (visible != isVisible()) {
      if (visible) show();
      else hide();
      super.setVisible(visible);
    }
  }
  
}