/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/OVTObject.java,v $
  Date:      $Date: 2000/04/11 15:42:26 $
  Version:   $Revision: 1.11 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 * OVTObject.java
 *
 * Created on March 15, 2000, 10:47 AM
 */
 
package ovt.object;

import ovt.beans.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import java.beans.*;
import javax.swing.*;

/** 
 * The root object for all OVT objects
 * @author  Mykola Khotyaintsev
 * @version 
 */
public class OVTObject implements NamedObject, OVTTreeNode, DescriptorsSource, Enableable {

  
  /** Holds value of property name. */
  protected String name = "no name";
  /** Holds value of property parent. */
  protected BasicObject parent = null;
  /** Holds value of property children. */
  protected Children children = null;
  /** Utility field used by bound properties. */
  protected OVTPropertyChangeSupport propertyChangeSupport = new OVTPropertyChangeSupport (this);
  /** Utility field used by constrained properties. */
  protected java.beans.VetoableChangeSupport vetoableChangeSupport = new java.beans.VetoableChangeSupport (this);
  /** Holds value of property descriptors. */
  protected Descriptors descriptors = null;
  /** Holds value of property valid. */
  protected boolean valid = false;
  /** Holds value of property enabled. */
  private boolean enabled = true;
  /** Holds value of property icon. */
  private Icon icon = null;
  
  /** Creates new OVTObject */
  public OVTObject() {
  }
  
  /** Creates new OVTObject */
  public OVTObject(String name) {
    setName(name);
  }
  
  /** Getter for property name.
   * @return Value of property name.
   */
  public String getName() {
    return name;
  }
  /** Setter for property name.
   * @param name New value of property name.
   */
  public void setName(String name) {
    this.name = name;
  }
  
  /** Getter for property parent.
   * @return Value of property parent.
   */
  public BasicObject getParent() {
    return parent;
  }
  
  /** Setter for property parent.
   * @param parent New value of property parent.
   */
  public void setParent(BasicObject parent) {
    this.parent = parent;
  }
  
  
  /** Add a PropertyChangeListener to the listener list.
   * @param l The listener to add.
   */
  public void addPropertyChangeListener(java.beans.PropertyChangeListener l) {
    propertyChangeSupport.addPropertyChangeListener (l);
  }

  /** Add a PropertyChangeListener to the listener list.
   * this is valid for JDK 1.1
   * @param l The listener to add.
   */
  public void addPropertyChangeListener(String property, java.beans.PropertyChangeListener l) {
    propertyChangeSupport.addPropertyChangeListener (property, l);
  }

  /** Removes a PropertyChangeListener from the listener list.
   * @param l The listener to remove.
   */
  public void removePropertyChangeListener(java.beans.PropertyChangeListener l) {
    propertyChangeSupport.removePropertyChangeListener (l);
  }
  /** Add a VetoableChangeListener to the listener list.
   * @param l The listener to add.
   */
  public void addVetoableChangeListener(java.beans.VetoableChangeListener l) {
    vetoableChangeSupport.addVetoableChangeListener (l);
  }
  /** Removes a VetoableChangeListener from the listener list.
   * @param l The listener to remove.
   */
  public void removeVetoableChangeListener(java.beans.VetoableChangeListener l) {
    vetoableChangeSupport.removeVetoableChangeListener (l);
  }
  
  /** Fires property change
   * 
   */
  protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
    propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
  }
  
  /** Getter for property children.
   * @return Value of property children.
   */
  public Children getChildren() {
    return children;
  }
  /** Setter for property children.
   * @param children New value of property children.
   *
   * @throws PropertyVetoException
   */
  public void setChildren(Children children) {
    Children oldChildren = this.children;
    this.children = children;
    propertyChangeSupport.firePropertyChange ("children", oldChildren, children);
  }
  
  
  
  /** Getter for property descriptors.
   * @return Value of property descriptors.
   */
  public Descriptors getDescriptors() {
    return descriptors;
  }
  /** Setter for property descriptors.
   * @param descriptors New value of property descriptors.
   */
  public void setDescriptors(Descriptors descriptors) {
    this.descriptors = descriptors;
  }
  
  public boolean isValid() {
    return valid;
  }
  /** Sets property valid to false.
   *
   */
  public void invalidate() {
    valid = false;
  }

  /** Getter for property enabled.
   * @return Value of property enabled.
   */
  public boolean isEnabled() {
    return enabled;
  }
  /** Setter for property enabled.
   * @param enabled New value of property enabled.
   */
  public void setEnabled(boolean enabled) {
    boolean oldValue = this.enabled;
    this.enabled = enabled;
    firePropertyChange("enabled", new Boolean(oldValue), new Boolean(enabled));
  }


  /** Getter for property icon.
   * @return Value of property icon.
   */
  public Icon getIcon() {
    return icon;
  }
  /** Setter for property icon.
   * @param icon New value of property icon.
   */
  public void setIcon(Icon icon) {
    this.icon = icon;
  }

}