/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/OrbitModule.java,v $
  Date:      $Date: 2000/04/11 15:42:26 $
  Version:   $Revision: 1.7 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object;

import vtk.*;

import ovt.*;
import ovt.beans.*;
import ovt.util.*;
import ovt.event.*;
import ovt.object.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import java.beans.*;
import java.util.*;
import javax.swing.*;

/**
 * This  that should be added
 * to { @link ovt.datatypes.Sat }. 
 * @author Mykola Khotyaintsev
 * @see ovt.datatypes.AbstractSatModule
 */
public class OrbitModule extends AbstractVisualSatModule {

protected vtkActor orbitActor = null;
public vtkActor actor = null; // vtkActors in each Coord. System

  public OrbitModule(Sat satellite) { 
    super(satellite, "Orbit", OVTCore.getImagesDir() + "orbit.gif");
    //initialize();
  }

  public void initialize() {
  // get default descriptors from VisualObject
    Descriptors descriptors = super.getDescriptors();
      try {
        // time
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("visible", this);
        pd.setDisplayName("Visibility");
        
        BasicPropertyEditor editor = new VisibilityEditor(pd);
        addPropertyChangeListener("visible", editor);
        pd.setPropertyEditor(editor);
        descriptors.put(pd);
        
      } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
        System.exit(0);
      }
  }
  
  protected boolean actorIsInRenderer() { 
    int positionInList = getRenderer().GetActors().IsItemPresent(getActor());
    return (positionInList != 0); 
  }

  public void show() {
    if (canBeVisible()) {
      getRenderer().AddActor(getActor());
    }
  }

  public void hide() {
    if (actor != null) { 
      getRenderer().RemoveActor(actor);
      actor = null;
    }
  }
  
  /*public void show() {
    if (canBeVisible() && !actorIsInRenderer()) getRenderer().AddActor(getActor());
  }

  public void hide() {
    if (actorIsInRenderer()) getRenderer().RemoveActor(getActor());
  }*/


 public void setVisible(boolean visible) {
    if (isVisible() != visible) {
      if (visible) show();
      else hide();
      super.setVisible(visible);
    }
  }

public void validate() {
  actor = null;
  valid = true;
}

  public void timeChanged(TimeEvent evt) {
    if (evt.getPropertyName().equals("time")) {
      invalidate();
      if (isVisible()) {
        hide();
        show();
      }
    }
  }
  
  public void coordinateSystemChanged(CoordinateSystemEvent evt) {
    System.out.println("Coord system changed.. I see...");
    invalidate();
    if (isVisible()) {
      hide();
      show();
    }
  }

  
  protected vtkActor getActor() {
    if (!isValid()) validate();
    
    if ( actor == null)  {
      int resolution = 10; // number of interpolated points between 2 normal points
      vtkPolyData profile = new vtkPolyData();
      vtkCellArray lines = new vtkCellArray();
      TrajectoryPoint tr;
      vtkCardinalSpline aSplineX = new vtkCardinalSpline();
      vtkCardinalSpline aSplineY = new vtkCardinalSpline();
      vtkCardinalSpline aSplineZ = new vtkCardinalSpline();

      double i = 0;
      double numberOfPoints = getTrajectory().size();
      Enumeration e = getTrajectory().elements();
      while (e.hasMoreElements()) {
        tr = (TrajectoryPoint)e.nextElement();
        double[] a = tr.get(getCS());
        aSplineX.AddPoint(i, a[0]);
        aSplineY.AddPoint(i, a[1]);
        aSplineZ.AddPoint(i, a[2]);
        i++;
      }
      
      vtkPoints points = new vtkPoints();
      
      double x,y,z;
      for (i=0; i<numberOfPoints; i+=1.0/resolution) {
        x = aSplineX.Evaluate(i);
        y = aSplineY.Evaluate(i);
        z = aSplineZ.Evaluate(i);
        points.InsertNextPoint(x, y, z);
      }
      
      lines.InsertNextCell(points.GetNumberOfPoints());
		
      for(int j=0; j<points.GetNumberOfPoints(); j++) 
		lines.InsertCellPoint(j);

	
	profile.SetPoints(points);
	profile.SetLines(lines);
	//profile.GetPointData().SetScalars(scalars);

	vtkLookupTable lut  = new vtkLookupTable();
	lut.SetHueRange(0.6667, 0);
	
	vtkPolyDataMapper mapper = new vtkPolyDataMapper();
	mapper.SetInput(profile);
	//mapper.SetScalarModeToUsePointData();
    //mapper.ScalarVisibilityOn();
	//mapper.SetScalarRange(minVelocity, maxVelocity);
	//mapper.SetLookupTable(lut);
	
      actor = new vtkActor();
      actor.SetMapper(mapper);
      actor.GetProperty().SetColor(1, 0, 0);
    }
    return actor;
  }

  
}


/*


  protected vtkActor getActor() {
    
    if (!isValid()) validate();
    
    if ( actor == null)  {
      vtkPolyData profile = new vtkPolyData();
      vtkPoints points = new vtkPoints();
      vtkCellArray lines = new vtkCellArray();
      vtkScalars scalars = new vtkScalars();
      TrajectoryPoint tr;

      Enumeration e = getTrajectory().elements();
	
	while (e.hasMoreElements()) {
		tr = (TrajectoryPoint)e.nextElement();
		double[] a = tr.get(getCS());
		points.InsertNextPoint(a[0],a[1],a[2]);
		scalars.InsertNextScalar(Vect.absv(tr.vei));
		//System.out.println(Vect.absv(tr.vei));
                //System.out.println(Time.toString(tr.mjd));
	}
	
	lines.InsertNextCell(getTrajectory().size());
		
	for(int i=0; i<getTrajectory().size(); i++) 
		lines.InsertCellPoint(i);

	
	profile.SetPoints(points);
        profile.Set
	profile.SetLines(lines);
	//profile.GetPointData().SetScalars(scalars);

	vtkLookupTable lut  = new vtkLookupTable();
	lut.SetHueRange(0.6667, 0);
	
	vtkPolyDataMapper mapper = new vtkPolyDataMapper();
	mapper.SetInput(profile);
	//mapper.SetScalarModeToUsePointData();
    //mapper.ScalarVisibilityOn();
	//mapper.SetScalarRange(minVelocity, maxVelocity);
	//mapper.SetLookupTable(lut);
	
      actor = new vtkActor();
      actor.SetMapper(mapper);
      actor.GetProperty().SetColor(1, 0, 0);
    }
    return actor;
  }

*/
/*
  protected vtkActor getActor() {
    
    if (!isValid()) validate();
    
    if ( actor == null)  {
      vtkPolyData polyData = new vtkPolyData();
      vtkPoints points = new vtkPoints();
      vtkCellArray lines = new vtkCellArray();
      vtkScalars scalars = new vtkScalars();
      vtkVectors vectors = new vtkVectors();
      TrajectoryPoint tr;
      Enumeration e = getTrajectory().elements();
	
	while (e.hasMoreElements()) {
		tr = (TrajectoryPoint)e.nextElement();
		double[] a = tr.get(CoordinateSystem.GEI);
		points.InsertNextPoint(a[0],a[1],a[2]);
		scalars.InsertNextScalar(Vect.absv(tr.vei));
                vectors.InsertNextVector(tr.vei[0], tr.vei[1], tr.vei[2]);
		//System.out.println(Vect.absv(tr.vei));
                //System.out.println(Time.toString(tr.mjd));
	}
	
	lines.InsertNextCell(getTrajectory().size());
		
	for(int i=0; i<getTrajectory().size(); i++) 
		lines.InsertCellPoint(i);

	polyData.SetPoints(points);
        polyData.GetPointData().SetVectors(vectors);
        polyData.SetLines(lines);
	//profile.GetPointData().SetScalars(scalars);

        
        vtkStreamLine streamLine = new vtkStreamLine();
        streamLine.SetInput(polyData);
        streamLine.SetIntegrationDirectionToIntegrateBothDirections();
        double[] s = getTrajectory().firstElement().get(CoordinateSystem.GEI);
        //streamLine.SetStartPosition(s[0],s[1],s[2]);
        streamLine.SetStartPosition(2,2,2);
        //streamLine.SetIntegrationStepLength(0.001);
        streamLine.SetStepLength(0.001);
	vtkLookupTable lut  = new vtkLookupTable();
	lut.SetHueRange(0.6667, 0);
	
	vtkPolyDataMapper mapper = new vtkPolyDataMapper();
	mapper.SetInput(streamLine.GetOutput());
	//mapper.SetScalarModeToUsePointData();
    //mapper.ScalarVisibilityOn();
	//mapper.SetScalarRange(minVelocity, maxVelocity);
	//mapper.SetLookupTable(lut);
	
      actor = new vtkActor();
      actor.SetMapper(mapper);
      actor.GetProperty().SetColor(1, 0, 0);
    }
    return actor;
  }


*/