/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/OrbitMonitorModule.java,v $
  Date:      $Date: 2000/06/13 20:52:34 $
  Version:   $Revision: 1.20 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object;

import ovt.*;
import ovt.gui.*;
import ovt.event.*;
import ovt.util.*;
import ovt.datatype.*;
import ovt.interfaces.*;
import ovt.mag.model.magnetopause.*;

import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class OrbitMonitorModule extends AbstractSatModule {

  protected JTextField x, y, z, radius, velocity, time, dist_to_mpause, spin;
  protected JTextField[] latitude = new JTextField[2];
  protected JTextField[] longitude = new JTextField[2];
  protected JLabel position, footprint;
  protected JDialog orbitMonitor = null;
  private DumpRecord dumpRecord = new DumpRecord();
  
public OrbitMonitorModule(Sat sat) {
  super(sat, "Orbit Monitor", OVTCore.getImagesDir()+"orbitmonitor.gif");
}

protected JDialog getOrbitMonitor() {
  if (orbitMonitor == null) {
    orbitMonitor = new JDialog();
    orbitMonitor.setTitle(getSat().getName());
    orbitMonitor.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    //orbitMonitor.addWindowListener(new WindowListener() {});
    setInterior(orbitMonitor.getContentPane());
    orbitMonitor.pack();
    orbitMonitor.setResizable(false);
  }
  return orbitMonitor;
}

public void setVisible(boolean visible) {
  getOrbitMonitor().setVisible(visible);
  if (visible) refresh();
  super.setVisible(visible);
}

/** Set the interior of container */
protected void setInterior(Container container) {
        
        GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(5, 2, 5, 2);
	c.anchor = GridBagConstraints.WEST;
	c.fill = GridBagConstraints.HORIZONTAL;
	container.setLayout(gridbag);

	// Create components

	footprint = new JLabel("Footprint at 100km [SMC]: ");
        for (int i=0; i>2; i++) {
	  latitude[i] = new JTextField("Latitude=50 ");
	  longitude[i] = new JTextField("Longitude=38 ");
        }
	time = new JTextField("2000-01-01 00:00");
	//time.setEditable(false);
	
	// Insert

	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	gridbag.setConstraints(time, c);
	container.add(time);
	
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = 1;
	JPanel comp = getPositionPanel();
	gridbag.setConstraints(comp, c);
	container.add(comp);
	
	c.gridx = 0;
	c.gridy = 2;
        c.gridwidth = 1;
	comp = getFootprintPanel();
	gridbag.setConstraints(comp, c);
	container.add(comp);
        
        c.gridx = 0;
	c.gridy = 3;
        c.gridwidth = 1;
	comp = getMagnetopauseDistancePanel();
	gridbag.setConstraints(comp, c);
	container.add(comp);
        
        c.gridx = 0;
	c.gridy = 4;
        c.gridwidth = 1;
	comp = getSpinPanel();
	gridbag.setConstraints(comp, c);
	container.add(comp);
}


protected JPanel getPositionPanel() {

	JPanel container = new JPanel();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
	//container.setBorder(BorderFactory.createEtchedBorder());
	position = new JLabel("Position [GSM]: ");
	x = new JTextField("X=-200.23 RE");
	y = new JTextField("X=-200.23 RE");
	z = new JTextField("X=-200.23 RE");
	radius = new JTextField("Radius=200.23 RE");
	velocity = new JTextField("Velocity=10 Km/s");

	container.add(position);
	container.add(x);
	container.add(y);
	container.add(z);
	container.add(velocity);
	container.add(radius);

	return container;
}


protected JPanel getFootprintPanel() {
	JPanel container = new JPanel();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
	
	// Create components
	
	footprint = new JLabel("Footprint at 100km [SMC], [CGM]");
        container.add(footprint);
        
        for (int i=0; i<2; i++) {
	  latitude[i] = new JTextField("Latitude=50 ");
	  longitude[i] = new JTextField("Longitude=38 ");
          container.add(latitude[i]);
          container.add(longitude[i]);
        }
	return container;
}


protected JPanel getMagnetopauseDistancePanel() {

	JPanel container = new JPanel();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
        
	JLabel title = new JLabel(new String(DumpRecord.strMP_DIST_LABEL));
	dist_to_mpause = new JTextField("r=200.23 RE");
	container.add(title);
	container.add(dist_to_mpause);
	return container;
}

protected JPanel getSpinPanel() { //added by kono

	JPanel container = new JPanel();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
        
	JLabel title = new JLabel(new String(DumpRecord.strSPIN_LABEL));
	spin = new JTextField("B = 20,  V = 31,  S = 12");
	container.add(title);
	container.add(spin);
	return container;
}

public void refresh() {
	refreshPositions();
        refreshSpin();
        //sat.dumper.writeDumpRecord(this.dumpRecord);
        //sat.dumper.close();
}

// is also executed from Sats

public void refreshPositions() {

    TrajectoryPoint trp = getTrajectoryPoint();

      String timetmp = Time.toString(getMjd());
      time.setText(timetmp);
      dumpRecord.setField(DumpRecord.TIME,timetmp);
      String posLabelName="Position ["+CoordinateSystem.getCoordSystem(getCS())+"]";
      position.setText(posLabelName);
      dumpRecord.setField(DumpRecord.POS_LABEL,posLabelName);

      String xs, ys, zs, rs, vs, d_to_mpause;
      double[] a;
      if (trp != null) {
	a = trp.get(getCS());
        xs = Utils.cuttedString(a[0],2)+ " RE";
        ys = Utils.cuttedString(a[1],2)+ " RE";
        zs = Utils.cuttedString(a[2],2);
        rs = Utils.cuttedString(trp.rr,2) + " RE";
        vs = Utils.cuttedString(Vect.absv(trp.vei),2) + " Km/sec";
        // distance to mpause
        double dist = Shue97.distance_to_magnetopause(trp.gsm, getMagProps().getSWP(getMjd()), getMagProps().getIMF(getMjd())[2]);
        // do not print distance to mpause, if this distance is > 10 Re
        if (dist < 10) d_to_mpause = Utils.cuttedString(dist,2) + " Re";
        else d_to_mpause = "> 10 Re";
        
      } else {
        xs = "none";
        ys = "none";
        zs = "none";
        rs = "none";
        vs = "none";
        d_to_mpause = "none";
      }

	x.setText("X=" + xs);
        dumpRecord.setField(DumpRecord.POSX,xs);
	y.setText("Y=" + ys);
        dumpRecord.setField(DumpRecord.POSY,ys);
	z.setText("Z=" + zs);
        dumpRecord.setField(DumpRecord.POSZ,zs);
	
	radius.setText("Radius=" + rs);
        dumpRecord.setField(DumpRecord.RAD,rs);
	velocity.setText("Velocity=" + vs);
        dumpRecord.setField(DumpRecord.VEL,vs);
        
        //footprint.setText("Footprint at 100km ["+CoordinateSystem.getCoordSystem(getPolarCS())+"], [CGM]");
        String fpStrLabel = "Footprint at 100km ["+CoordinateSystem.getCoordSystem(getPolarCS())+"]";
        footprint.setText(fpStrLabel);
        dumpRecord.setField(DumpRecord.FOOT_LABEL,fpStrLabel);
        MagPoint[] magPoints = getMagFootprints(getMjd());
        MagPoint mp;
        double[] b, inCGM;
        String latitudeTitle = "", latCGM=""; //added by kono
        String longitudeTitle = "", mltCGM="";
        for (int i=0; i<2; i++) {
          mp = magPoints[i];
          if (mp != null) {
            a = getTrans(getMjd()).gsm_trans_matrix(getPolarCS()).multiply(mp.gsm);
	    if (getPolarCS() == CoordinateSystem.GEO) {
              /*//added by kono
              inCGM=Trans.corrgma(getMagProps(),getMjd(),a,100.0);
              latCGM = ", " + Utils.cuttedString(inCGM[0]*180.0/Math.PI, 2)+" deg";
              mltCGM = ", " + Utils.cuttedString(inCGM[2]*180.0/Math.PI, 2)+" deg";
              */
              b = Trans.geo2spherical(a);
              latitudeTitle = "latitude = "+Utils.cuttedString(b[0], 2)+" deg";
	      longitudeTitle = "longitude = "+Utils.cuttedString(b[1], 2)+" deg";
            } else if (getPolarCS() == CoordinateSystem.SM) {
              /*//added by kono
              double[] aa=getTrans(getMjd()).sm_geo_trans_matrix().multiply(a);
              inCGM=Trans.corrgma(getMagProps(),getMjd(),aa,100.0);
              latCGM = ", " + Utils.cuttedString(inCGM[0]*180.0/Math.PI, 2)+" deg";
              mltCGM = ", " + Utils.cuttedString(inCGM[2]*12.0/Math.PI, 2)+" h";
              */
              b = Trans.sm2spherical(a);
              latitudeTitle = "mlat = "+Utils.cuttedString(b[0], 2)+" deg";
	      longitudeTitle = "mlt = "+Utils.cuttedString(b[1], 2) +" h";
            }
          } else {
            latitudeTitle = "none";
	    longitudeTitle = "none";
            
            //added by kono
            latCGM="";
            mltCGM="";
          }
          latitude[i].setText(latitudeTitle+latCGM);
	  longitude[i].setText(longitudeTitle+mltCGM);
        }
        dumpRecord.setField(DumpRecord.LAT1,latitude[0].getText());
        dumpRecord.setField(DumpRecord.LON1,longitude[0].getText());
        dumpRecord.setField(DumpRecord.LAT2,latitude[1].getText());
        dumpRecord.setField(DumpRecord.LON2,longitude[1].getText());
        
        dist_to_mpause.setText(d_to_mpause);
        dumpRecord.setField(DumpRecord.MP_DIST,d_to_mpause);
}

public void refreshSpin() {
  double[] angles=this.sat.getSpinAngles();
  if(angles==null)
     spin.setText("no data");
  else {
     String[] bvs={"B = ",",  V = ",",  S = "};
     String bvsstr=new String("");
     for(int i=0;i<3;++i){
        angles[i]*=180.0/Math.PI;
        bvsstr=bvsstr.concat(bvs[i]);
        if(Math.abs(angles[i])<=360.0)
           bvsstr=bvsstr.concat(new Integer((int)Math.round(angles[i])).toString());
        else
           bvsstr=bvsstr.concat("NaN");
     }
     spin.setText(bvsstr);
     dumpRecord.setField(DumpRecord.SPIN,bvsstr);
     //spin.setText("&B = "+(int)angles[0]+", &V = "+(int)angles[1]+", &S = "+(int)angles[2]);
  }
}

public void timeChanged(TimeEvent e) { 
  if (isVisible()) refresh();
}

public void coordinateSystemChanged(CoordinateSystemEvent evt) { 
  if (isVisible()) refresh();
}

public DumpRecord getDumpRecord(){
  return (DumpRecord)this.dumpRecord.clone();
}

public String getName() {
  //System.out.println(getSat().getName()+".OrbitMonitor");
  return getSat().getName()+".OrbitMonitor";
}
}
