/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/Sat.java,v $
  Date:      $Date: 2000/06/14 12:06:00 $
  Version:   $Revision: 1.55 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object;

import vtk.*;

import ovt.*;
import ovt.beans.*;
import ovt.interfaces.*;
import ovt.datatype.*;
import ovt.gui.*;
import ovt.mag.*;
import ovt.util.*;
import ovt.event.*;

import java.beans.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

public class Sat extends VisualObject implements CoordinateSystemChangeListener, 
  TimeChangeListener, MagPropsChangeListener {

  
  protected String orbitFileName;

  public double Hourfwd = 4.40;
// [NSAT]={4.40, 57.0, 66.0, 82.0, 24.0};

/*  orbital period hours = 2.7645e-6*A*sqrt(A) A(km)=semimajor axis   */
//   public double MapRadius = 8.0;   /* radius for mapping cusp etc. */

/**
 * Satellite's trajectory
 */
  protected Trajectory tra = new Trajectory();
  double maxVelocity = 0, minVelocity = Double.MAX_VALUE;
  private Children modules = new Children();

  //added by kono:
  protected Tle tleFile;
  //0 - TLE data file; 1,2,3,4 - CLUSTER data file; 5 - CLUSTER other
  private byte satType=0;
  protected String spinFileName;
  protected SpinData spinData;
  public Dumper dumper;
  private DumperDialog dumperDialog = null;
  private BasicPropertyEditor editor;
  
  protected MainFieldlineModule mainFieldlineModule;
  
  protected MagFootprintModule magFootprintModule;

    /** Holds value of property firstDataMjd. */
  private double firstDataMjd;
  
  /** Holds value of property lastDataMjd. */
  private double lastDataMjd;
  
  private boolean dumperVisible = false;
  
  public Sat(String name, String orbitFileName, OVTCore core)
          throws IOException {
    super(core, name);
    try {
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("dumperVisible", this);
        pd.setDisplayName("dumpererVisible");
        
        editor = new BooleanEditor(pd, MenuPropertyEditor.SWITCH);
        editor.setTags(new String[]{"Dump data", "Hide dumper"});
        addPropertyChangeListener("dumperVisible", editor);
        pd.setPropertyEditor(editor);
        getDescriptors().put(pd);
        //System.out.println("SUKA!!!");
        
  } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
  }
    initialize(name, orbitFileName, true);
  }
  
  public Sat(String name, String orbitFileName, OVTCore core, boolean setupDataPeriod)
          throws IOException {
    super(core, name);
    try {
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("dumperVisible", this);
        pd.setDisplayName("dumpererVisible");
        
        editor = new BooleanEditor(pd, MenuPropertyEditor.SWITCH);
        editor.setTags(new String[]{"Dump data", "Hide dumper"});
        addPropertyChangeListener("dumperVisible", editor);
        pd.setPropertyEditor(editor);
        getDescriptors().put(pd);
        //System.out.println("SUKA!!!");
        
  } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
  }
    initialize(name, orbitFileName, setupDataPeriod);
  }

  protected void initialize(String name, String orbitFileName, boolean setupDataPeriod) 
          throws IOException{
    this.orbitFileName = orbitFileName;
    
    //following paragraph of code added by kono
    String tmps=new String(orbitFileName);
    int tmpi=tmps.lastIndexOf('.');
    this.spinFileName=new String(tmps.substring(0,tmpi)+".spin");
    this.spinData=new SpinData(spinFileName);
    if (orbitFileName.endsWith(".tle")) {              //Def. type of data?
       satType = 0;                                    //TLE satellite
       tleFile=new Tle(this.orbitFileName);
       firstDataMjd = tleFile.getFirstMjd();
       lastDataMjd = tleFile.getLastMjd();
    } else {                                           //CLUSTER satellite
       char type=orbitFileName.charAt(orbitFileName.length()-4);
       switch(type){
          case '1': satType = 1; break;
          case '2': satType = 2; break;
          case '3': satType = 3; break;
          case '4': satType = 4; break;
          default: satType = 5;       //error!!!
       }
    }

    setIcon(new ImageIcon(OVTCore.getImagesDir() + "satellite.gif"));
    // create sat modules and put them to children
    
    AbstractSatModule satModule = new SatelliteModule(this);
    addPropertyChangeListener(satModule);
    modules.put(satModule);
    
    satModule = new OrbitModule(this);
    addPropertyChangeListener(satModule);
    modules.put(satModule);
    
    satModule = new OrbitMonitorModule(this);
    addPropertyChangeListener(satModule);
    modules.put(satModule);
    
    mainFieldlineModule = new MainFieldlineModule(this);
    addPropertyChangeListener(mainFieldlineModule);
    modules.put(mainFieldlineModule);
    
        
    // create mainFootprintModule
    AbstractSatModule mainFootprintModule = new AbstractSatModule(this, "Footprints", getCore().getImagesDir()+"footprints.gif");
    // create magFootprintModule
    magFootprintModule = new MagFootprintModule(mainFootprintModule);
    // set mainFootprintModule children
    Children mfpmChildren = new Children();
    // set magFootprintModule
    mfpmChildren.put(magFootprintModule);
    mainFootprintModule.addPropertyChangeListener(magFootprintModule);
    mainFootprintModule.setChildren(mfpmChildren);
    // add mainFootprintModule
    addPropertyChangeListener(mainFootprintModule);
    modules.put(mainFootprintModule);
    
    dumper = new Dumper();
    dumperDialog = new DumperDialog(getCore().XYZwin,false,this);
    
    //modules.put("OrbitMonitor",new OrbitMonitorModule(this, getCore(), getVW()));
    if (setupDataPeriod) setupDataPeriod();
    updateEnabled();
  }
  
  //added by kono
  /**
   *
   *@return null or spin vector in current CS (abs. value of vector is a spin rate)
   */
  public double[] getSpinVectorRate(){
     if(this.isSpinAvailable()){
        double[] geiSpin=spinData.getSpinVect(getMjd());
        if(geiSpin==null)
           return null;
        Trans trans=this.getTrans(getMjd());
        Matrix3x3 mtrx=trans.gei_gsm_trans_matrix(); //GEI -> GSM
        double[] gsmSpin=mtrx.multiply(geiSpin);
        mtrx=trans.gsm_trans_matrix(getCS());        //GSM -> current CS
        return mtrx.multiply(gsmSpin);               // result in current CS
     } else return null;
  }
  
  //added by kono
  /**
   *
   *@return null or angles between spin & B(mag. field), V(velocity), S(sun) vectors
   */
  public double[] getSpinAngles(){
     double mjdx=getMjd();
     double[] geiSpinVect=spinData.getSpinVect(mjdx);  //spin vect in GEI
     if(geiSpinVect==null)
        return null;
     GeiAndVei geivei;
     try {
        geivei=this.getSatPos(mjdx);                   //sat pos&vel in GEI CS
     } catch (IOException e){
        return null;
     }
     Trans trans=this.getTrans(mjdx);
     Matrix3x3 mtrx=trans.gei_gsm_trans_matrix();      //GEI -> GSM matrix
    
     double[] gsmSpinVect=mtrx.multiply(geiSpinVect);  //spin vect in GSM
     double[] gsmPos=mtrx.multiply(geivei.gei);        //sat position in GSM
     double[] gsmVel=mtrx.multiply(geivei.vei);        //sat velocity in GSM
     double[] gsmB=this.getMagProps().bv(gsmPos,mjdx); // B vector in GSM
     double[] geiES=Utils.sunmjd(mjdx);                //Earth->Sun vect in GEI
     double[] gsmES=mtrx.multiply(geiES);              //Earth->Sun vect in GSM
     double[] gsmSS=Vect.sub(gsmES,gsmPos);            //Sat->Sun vect in GSM

     Vect.normf(gsmB,1.0);
     Vect.normf(gsmSS,1.0);
     Vect.normf(gsmVel,1.0);
     Vect.normf(gsmSpinVect,1.0);
    
     double[] angles=new double[3];
     angles[0]=Vect.angleOf2vect(gsmSpinVect, gsmB);   //angle between Sp & B
     angles[1]=Vect.angleOf2vect(gsmSpinVect, gsmVel); //angle between Sp & V
     angles[2]=Vect.angleOf2vect(gsmSpinVect, gsmSS);  //angle between Sp & SunDir

     return angles;  //all in radians!!!, kono
  }
  
  //added by kono
  public boolean isSpinAvailable(){
    return spinData.isAvailable();
  }
  
  //added by kono
  public byte getSatType(){
     return satType;
  }
  
  public String getOrbitFileName() {
    return orbitFileName;
  }
  
    /** Returns the time for the first data entry in OrbitFile
   * @return The time for the first data in OrbitFile
   */
  public double getFirstDataMjd() {
    return firstDataMjd;
  }
  
  /** Returns the time for the last data entry in OrbitFile
   * @return Value of property lastDataMjd.
   */
  public double getLastDataMjd() {
    return lastDataMjd;
/*    try {
       return Tle.getLastMjd(this.orbitFileName);
    } catch (IOException e){
       System.err.println(e.toString());
       return 1000000;
    }*/
  }
  
  /** aa
   * @return Value of property timeSet...
   */
  public TimeSet getTimeSet() {
    return getCore().getTimeSettings().getTime();
  }
  

  /** Getter for property startMjd.
   * @return Value of property startMjd.
   */
  public double getStartMjd() {
    return getTimeSet().getStartMjd();
  }
  
  /** Getter for property stopMjd.
   * @return Value of property stopMjd.
   */
  public double getStopMjd() {
    return getTimeSet().getStopMjd();
  }
  
  /** Getter for property stepMjd.
   * @return Value of property stepMjd.
   */
  public double getStepMjd() {
    return getTimeSet().getStepMjd();
  }
  
  /** Sets up Sat's first data entry time */ 
  protected void setupDataPeriod() throws IOException {
    int ymd, hms, ms;
    double  sma = 0, periodh, mjd;
    int k;
    Hourfwd = 24.0;
    ymd = 800101;
    hms = ms = 0;
//    double res[] = getFirstMjd(getOrbitFileName()); // satorbit.c
        
//    firstDataMjd = res[0];
    
//    sma = res[1];
    sma = 7000.0;       // to be corrected by kono from koko (satorbit.c)
    periodh = 2.764e-6 * sma * Math.sqrt(sma);
    k = (int)periodh * 60;
    k = (k / 10 )* 10;
    periodh = (double) k / 60.0;
    Hourfwd = periodh;
  }

// modification of fill_traj from suppr.c

  protected void updateTrajectory() {
    System.out.print("Computing orbit for " + getName()+" ... ");
    int i, n, ok, maxzi;
    double  gei[] = new double[3], vei[] = new double[3], geo[] = new double[3],  gsm[] = new double[3], gse[] = new double[3];
    int  plus = 1;
    double  mjd, model, dist_bs, dist_mp, dist_cu, maxz;
    double altf,mlt;

    double ClatMax=0.0;  // max-corrected-latitude in orbit (subpr.c line: 40)
    double velocity;
    ok=1;
    maxz=0.0;
    maxzi=0;
    ClatMax=0.0;
    Trans trans;
    TrajectoryPoint tr;
    GeiAndVei geiAndVei;
    tra.clear();


    //System.out.println(new TimeSet(getStartMjd(), getStopMjd(), getStepMjd()));
    
    double[] timeMap = getTimeSet().getTimeMap();

    try {
     for (int index=0; index<timeMap.length; index++){
        mjd = timeMap[index];
        tr = new TrajectoryPoint();
        tr.mjd = mjd;
        // read the sat location
//        geiAndVei = getSatPos(getOrbitFileName(), mjd);
        //geiAndVei = tleFile.getSatPos(mjd);
        geiAndVei = getSatPos(mjd);
        
        gei = geiAndVei.gei;
        vei = geiAndVei.vei;
        //System.out.println("loaded pos:" + gei[0]+"\t"+gei[1]+"\t"+gei[2]);
        //System.out.println(Time.toString(mjd));
        velocity = Vect.absv(vei);
        if (velocity > maxVelocity) maxVelocity = velocity;
        if (velocity < minVelocity) minVelocity = velocity;
        
        for (i = 0; i < 3; i++) gei[i] /= Const.RE;
        
        // get transformation class
        trans = getTrans(mjd);
        // Transform gei to geo
        geo = trans.gei2geo(gei);
        // Transform geo to gsm
        gsm = trans.geo2gsm(geo);
        // Transform gei to gse
        gse = trans.gei2gse(gei);
        
        tr.rr = Vect.absv(gei);
        tr.gei = gei;
        tr.vei = vei;
        tr.geo = geo;
        tr.gsm = gsm;
        tr.gse = gse;
        tr.sm = trans.gsm2sm(tr.gsm);
        
        /*tr.gei = (double[])gei.clone();
        tr.vei = (double[])vei.clone();
      tr.geo = (double[])geo.clone();
      tr.gsm = (double[])gsm.clone();
      tr.gse = (double[])gse.clone();*/
        tr.fsm = new double[3];
        tr.fgeo = new double[3];
        tr.fsmc = new double[3];
        tr.efoot_gsm = new double[3];
        tr.foot_cor = new double[2][3];
        tr.foot_geo = new double[2][3];
        for (i = 0; i < 3; i++) {
                tr.fsm[i] = -99.0;
          tr.fgeo[i] = -99.0;
          tr.fsmc[i] = -99.0;
        tr.efoot_gsm[i]=-99.0;
        tr.foot_cor[0][i]=-99.0;
        tr.foot_geo[0][i]=-99.0;
        tr.foot_cor[1][i]=-99.0;
        tr.foot_geo[1][i]=-99.0;
              /*fgsm[i] = 99.0;*/
        }
      //tr.dist_bs = Subpr.getDistanceFromBsh(tr.gsm, getCore().models.Bsub); 
      //setDistanceFromMps(tr);
      //System.out.println(tr.dist_mp+"\t"+tr.dist_cu);
        tr.dist_eq = tr.gsm[2];       // to be corrected 
        if (tr.dist_mp <  0.5) {
      // To be corrected!!!
      //traceTrajectory(tr);
      //System.out.println(tr.foot_geo[0][0]);
        }
        tra.put(tr);
     }
    } catch (IOException e2) {
       getCore().sendErrorMessage(e2);
       //e2.printStackTrace();
    }

    System.out.println("Done.");
    System.out.println(getName() + "'s trajectory has " + tra.size() + " points"); 
    
    // System.out.println("max mlat = " + ClatMax + "\n");
    valid = true;
  }

// ------------------------------------------------------

  /** Returns Sat's Trajectory */
  public Trajectory getTrajectory() {
  if (!isValid()) validate();
  return tra;
  }

  /** Returns current Sat's TrajectoryPoint */
  public TrajectoryPoint getTrajectoryPoint() {
    return getTrajectory().get(getMjd());
  }

  /** Returns Sat's position in current CS */
  public double[] getPosition() {
    TrajectoryPoint p = getTrajectoryPoint();
    if (p != null) return p.get(getCS());
    else return null;
  }
  
  public FootprintCollection[] getMagFootprintCollection() {
    return magFootprintModule.getFootprintCollection();
  }
  
  
  public MagPoint[] getMagFootprints(double mjd) {
    FootprintCollection[] fpc =  getMagFootprintCollection();
    return new MagPoint[]{fpc[0].get(mjd), fpc[1].get(mjd)};
  }

  public final boolean canBeVisible()
  { return (getTrajectoryPoint() != null); }
/*
 public double[] getPosition() 
  { return getTrajectoryPoint().getCoord(getCore().getCoordSystem()); }
*/


  //--- FieldLine staff------
  
  protected FieldlineCollection getFieldlineCollection(int type) {
    return mainFieldlineModule.getFieldlineCollection(type);
  }
  
  protected Fieldline getFieldline(int type, double mjd) {
    return mainFieldlineModule.getFieldline(type, mjd);
  }
  
  public void validate() {
    updateTrajectory();
    valid = true;
  }



  


//-------------------------------------
// Get gei[3] and vei[3] using Sat.file and time.
//public static native int getSatPosJNI(String filename, double mjd, double[] gei, double[] vei);


 /** get S/C position and velocity
 * @param filename Orbit Data File
 * @param mjd Time
 * @throws IOException
 * @return S/C position and veocity
 */
public GeiAndVei getSatPos(double mjd) throws IOException {
  GeiAndVei res = new GeiAndVei();

  try{
     res=tleFile.getSatPos(mjd);
  } catch (IOException e){
     throw new IOException(e.toString());
  }
/*  int code = getSatPosJNI(filename, mjd, res.gei, res.vei);
  if (code != 0) throw new IOException("IOException with file " + filename);
*/
  return res;
}

// Output: res[0] = launchMjd, res[1] = sma
//public native void get_first_mjd(String filename, double[] res);


/** Returns time for the first data in orbit file
 * @depricated since 0.001
 */
/*public double[] getFirstMjd(String orbitFileName) throws IOException {
  double[] res = new double[2];
  get_first_mjd(orbitFileName, res);
  
  //if (code =! 0) throw new IOException("IOException with file " + filename);
  return res;
}*/


/*  JNI to distance_from_mps (d_from_mps) subpr.c
    output: 
    [0] - dmp - distance from magnetopause (negative inside)
    [1] - dcusp - distance from cusp (negative for south)
*/

/*
public native double[] distanceFromMps(double[] gsm, double mjd, 
                double model); 


public void setDistanceFromMps(TrajectoryPoint tr) {
  double[] res = distanceFromMps(tr.gsm, tr.mjd, 
                       getCore().getMagProps().model);
  tr.dist_mp = res[0];
  tr.dist_cu = res[1];
}
*/



/*
public void timeChanged(TimeEvent te) {

  if (te.getStateChange() == TimeEvent.PERIOD) {
    invalidate();
    validate();
  }

  Enumeration e = modules.elements();
  AbstractSatModule satModule;
  while(e.hasMoreElements()) {
    satModule = (AbstractSatModule)e.nextElement();
    satModule.timeChanged(te);
  }
  
  if (te.getStateChange() == TimeEvent.PERIOD) getCore().getVW().Render();
}
*/

  public void timeChanged(TimeEvent evt) {
    
    if (evt.getPropertyName().equals("time")) {
      // current time and time period have changed
      updateEnabled();
      invalidate();
    }
    
    // Tell children about time change
    Enumeration e = getChildren().elements();
    while (e.hasMoreElements()) {
      try {
       ((TimeChangeListener)(e.nextElement())).timeChanged(evt);
      } catch (ClassCastException e2) {
        System.out.println("this module doesn't care about time..");
      }
    }
  }

  public void updateEnabled() {
    TimeSet timeSet = getCore().getTimeSettings().getTime();
    double globalStart = timeSet.getStartMjd();
    double globalStop = timeSet.getStopMjd();
    // global time period shoud be inside Sat's data period
      if (getFirstDataMjd() <= globalStart  &&  getLastDataMjd() >= globalStop) {
        this.editor.setEnabled(true);
        if (!isEnabled())
          setEnabled(true);
      } else {
        // torbets
        this.editor.setEnabled(false);
        if (isEnabled())
          setEnabled(false);
      }
  }
   public Children getChildren() {
    return modules;
  }

  /*// for native methods
  static {
    System.loadLibrary("ovt2g");
  }*/

  public void coordinateSystemChanged(CoordinateSystemEvent evt) {
    Enumeration e = getChildren().elements();
    while (e.hasMoreElements()) {
      try {
       ((CoordinateSystemChangeListener)(e.nextElement())).coordinateSystemChanged(evt);
      } catch (ClassCastException e2) {
        System.out.println("this module doesn't care about cs..");
      }
    }
  }
  
  public void magPropsChanged(MagPropsEvent evt) {
    mainFieldlineModule.magPropsChanged(evt);
    magFootprintModule.magPropsChanged(evt);
  }


public boolean isDumperVisible(){
  return dumperVisible;
}

public void setDumperVisible(boolean dumperVisible) {
    boolean oldDumperVisible = this.dumperVisible;
    this.dumperVisible = dumperVisible;
    propertyChangeSupport.firePropertyChange ("dumperVisible", new Boolean (oldDumperVisible), new Boolean (dumperVisible));
    dumperDialog.setVisible(dumperVisible);
  }

  public void writeDumpRecord() {
    OrbitMonitorModule om = (OrbitMonitorModule) modules.get(getName()+".OrbitMonitor");
    dumper.writeDumpRecord(om.getDumpRecord());
  }
  public void setupDumper(java.lang.String file,boolean pos,boolean fp,boolean mp,boolean spin,boolean auto) {
    dumper.setupDump(file, pos, fp, mp, spin, auto);
  }
  public void dumperClose() {
    dumper.close();
  }
  
  /*private void setEnabled(boolean x){
    super(x);
    this.editor.setEnabled(x);
  }*/
}