/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/SatelliteModule.java,v $
  Date:      $Date: 2000/06/06 12:43:42 $
  Version:   $Revision: 1.15 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object;

import vtk.*;

import ovt.*;
import ovt.beans.*;
import ovt.mag.*;
import ovt.util.*;
import ovt.event.*;
import ovt.datatype.*;
import ovt.interfaces.*;

import java.beans.*;
import javax.swing.*;

/**
 * This  that should be added
 * to { @link ovt.datatypes.Sat }. 
 * @author Mykola Khotyaintsev
 * @see ovt.datatypes.AbstractSatModule
 */
public class SatelliteModule extends AbstractVisualSatModule {

protected vtkActor satActor = null;

//added by kono
protected vtkActor spinActor = null;
protected double[] spinActorPos = new double[3];
private float satRadius=0.03F;

public SatelliteModule(Sat satellite) {
  super(satellite, "Sat");
  setIcon(new ImageIcon(OVTCore.getImagesDir() + "satellite.gif"));
}

  //added by kono
  public float getSatRadius(){
    return satRadius;
  }
  
  public void setSatRadius(float sR){
    satRadius = sR;
    update();
  }
  
  protected vtkActor getActor() {
    if (satActor == null)
       satActor = createSatActor();
    return satActor;
  }

  public boolean isSpinActorAvailable(){
     return super.sat.isSpinAvailable();
  }

  protected boolean actorIsInRenderer() {
    int positionInList = getRenderer().GetActors().IsItemPresent(getActor());
   // System.out.println("SatelliteModule. actor's position = " + positionInList);
    return (positionInList != 0);
  }
  
  protected void show() {
    if (canBeVisible() && !actorIsInRenderer()) { // 
      //System.out.println("Showing actor!!! ");
      getRenderer().AddActor(getActor());
    }
  }

  protected void hide() {
    if (actorIsInRenderer()){
       getRenderer().RemoveActor(getActor());
    }
  }
  
  protected void update()
  { update(isVisible()); } 
  
  protected void update(boolean visible) {
    if (visible) {
      double[] a = getPosition();
      if (a != null) {
        // update actors position
        //getActor().SetPosition(a[0], a[1], a[2]);
        hide();
        satActor=createSatActor();
        satActor.SetPosition(a[0], a[1], a[2]);

        //System.out.println(getName() + " pos: " + a[0]+"\t"+a[1]+"\t"+a[2]);
        // add actor if it is not in renderer
        show();
      } 
    } else { // no position data availible - hide actor
      //System.out.println("Trying to hide actor!!! ");
      hide();
    }
  }
  
  private vtkActor createSatActor() {
     
    vtkActor actor=createSatAsArrow();
     if(actor==null)
        actor=createSatAsSphere();
    
    //Define color from satType
    byte type=sat.getSatType();
    switch(type){
       case 1: actor.GetProperty().SetColor(1,0,0); break;     //red
       case 2: actor.GetProperty().SetColor(0,1,0); break;     //green
       case 3: actor.GetProperty().SetColor(1,1,0); break;     //yellow
       case 4: actor.GetProperty().SetColor(1,0,1); break;     //pink
       default: actor.GetProperty().SetColor(0,0,1);           //blue
    }
    return actor;
}

 private vtkActor createSatAsSphere(){
   // create sphere geometry
   vtkActor actor = new vtkActor();
   vtkSphereSource sphere = new vtkSphereSource();
   sphere.SetRadius(getSatRadius());

   sphere.SetThetaResolution(8);
   sphere.SetPhiResolution(8);

   // map to graphics library
   vtkPolyDataMapper map = new vtkPolyDataMapper();
   map.SetInput(sphere.GetOutput());

   // actor coordinates geometry, properties, transformation
   actor.SetMapper(map);
   return actor;
 }

  private vtkActor createSatAsArrow(){
     if(this.isSpinActorAvailable()){
        double[] spinVect=super.sat.getSpinVectorRate();
        if(spinVect!=null){
           vtkActor actor;
           vtkPolyData polyData = new vtkPolyData();
           vtkPoints points = new vtkPoints();
           vtkVectors vectors = new vtkVectors();
          
           //Vect.normf(spinVect,2.0*satRadius);
          
           /*double[] x=getPosition(), satPosx=new double[3];
           for(int i=0;i<3;++i) satPosx[i]=x[i];
           
           points.InsertNextPoint(satPosx[0], satPosx[1], satPosx[2]);*/
           points.InsertNextPoint(0, 0, 0);
           vectors.InsertNextVector(spinVect[0],spinVect[1],spinVect[2]);
          
           polyData.SetPoints(points);
           polyData.GetPointData().SetVectors(vectors);
          
           vtkConeSource cone = new vtkConeSource();
           cone.SetResolution(2);
          
           vtkGlyph3D glyph = new vtkGlyph3D();
           glyph.SetInput(polyData);
           glyph.SetSource(cone.GetOutput());
           glyph.SetVectorModeToUseVector();
           glyph.OrientOn();
          
           glyph.SetScaleFactor(0.04);
      
  //vtkLookupTable lut  = new vtkLookupTable();
  //lut.SetHueRange(0.6667, 0);
	
           vtkPolyDataMapper mapper = new vtkPolyDataMapper();
           mapper.SetInput(glyph.GetOutput());
  //mapper.SetScalarModeToUsePointData();
  //mapper.ScalarVisibilityOn();
  //mapper.SetScalarRange(getMjd()*0.8, getMjd()*1.2);
  //mapper.SetLookupTable(lut);
	
           actor = new vtkActor();
           actor.SetMapper(mapper);
          
           return actor;
        }
     }
     return null;
  }

  public void setVisible(boolean visible) {
    if (isVisible() != visible) {
      update(visible);
      super.setVisible(visible);
    }
  }

  public void timeChanged(TimeEvent e) {
    if (isVisible()) update();
  }
  
  public void coordinateSystemChanged(CoordinateSystemEvent evt) {
    if (isVisible()) update();
  }

}
