/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/TimeSettings.java,v $
  Date:      $Date: 2000/06/08 13:21:56 $
  Version:   $Revision: 1.13 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 * TimeSettings.java
 *
 * Created on February 28, 2000, 11:00 AM
 */
 
package ovt.object;

import ovt.*;
import ovt.beans.*;
import ovt.event.*;
import ovt.datatype.*;
import ovt.interfaces.*;
import ovt.object.editor.*;

import java.beans.*;

import java.util.*;
import java.lang.reflect.*;

/** 
 *
 * @author  mykola
 * @version 
 */
public class TimeSettings extends VisualObject implements java.io.Serializable,
                VetoableChangeListener {

  public static final String PROP_TIME = "time";
  public static final String PROP_CURRENT_MJD = "currentMjd";

  private OVTPropertyChangeSupport propertySupport = new OVTPropertyChangeSupport ( this );

  /** Utility field used by constrained properties. */
  private java.beans.VetoableChangeSupport vetoableChangeSupport = new java.beans.VetoableChangeSupport (this);
  
  private TimeChangeSupport timeChangeSupport = new TimeChangeSupport(this);
  /** Holds value of property startMjd. */
  private double startMjd;
  /** Holds value of property stopMjd. */
  private double stopMjd;
  /** Holds value of property stepMjd. */
  private double stepMjd;
  /** Holds value of property currentMjd. */
  double currentMjd = 0;
  
  private TimeSet time = new TimeSet(Time.getMjd("2000-01-01 00:00:00"), 
                                        Time.getMjd("2000-01-02 00:00:00"), 
                                        MinutesAndSeconds.getInDays("20:00"));
  private TimeSet futureTime = new TimeSet();
  private Vector timeMap = new Vector();
  
  private Object parent = null;
  private Descriptors propertyDescriptors = null;
  
  /** Creates new TimeSettings */
  public TimeSettings(OVTCore core) {
    super(core, "Time Settings");
    addVetoableChangeListener(this);
    initialize();
  }

  public void addTimeChangeListener (TimeChangeListener listener) {
    timeChangeSupport.addTimeChangeListener (listener);
  }

  public void removeTimeChangeListener (TimeChangeListener listener) {
    timeChangeSupport.removeTimeChangeListener (listener);
  }

  
  public void addPropertyChangeListener (PropertyChangeListener listener) {
    propertySupport.addPropertyChangeListener (listener);
  }
  
  public void addPropertyChangeListener (String property, PropertyChangeListener listener) {
    propertySupport.addPropertyChangeListener (property, listener);
  }
  
  public void removePropertyChangeListener (PropertyChangeListener listener) {
    propertySupport.removePropertyChangeListener (listener);
  }

  /** Add a VetoableChangeListener to the listener list.
   * @param l The listener to add.
   */
  public void addVetoableChangeListener(java.beans.VetoableChangeListener l) {
    vetoableChangeSupport.addVetoableChangeListener (l);
  }
  /** Removes a VetoableChangeListener from the listener list.
   * @param l The listener to remove.
   */
  public void removeVetoableChangeListener(java.beans.VetoableChangeListener l) {
    vetoableChangeSupport.removeVetoableChangeListener (l);
  }
  
  public synchronized void setTime(TimeSet newTime) throws PropertyVetoException {
    if (newTime.isReady()) {
      TimeSet oldTime = this.time;
      if (oldTime.equals(newTime)) return; // nothing's changed
      try {
        vetoableChangeSupport.fireVetoableChange(PROP_TIME, oldTime, newTime);
      } catch (PropertyVetoException e) {
        newTime.clear();
        // pass the exception for the further snalysis.
        throw e;
      }
      // the time is valid
      this.time = newTime;
      //time.setCurrentMjd(time.getStartMjd());
      
      setTimeMap(time);
      
      TimeEvent timeEvent = new TimeEvent(this, PROP_TIME, oldTime, time, getTimeMap());
      if (OVTCore.DEBUG > 0) System.out.println("TimeSettings. firePropertyChange 'time'");
      propertySupport.firePropertyChange(timeEvent);
      if (OVTCore.DEBUG > 0) System.out.println("TimeSettings.fireTimeChange");
      timeChangeSupport.fireTimeChange(timeEvent);
      // adjust newTime for future this should be in the very end!
      newTime = oldTime;
      newTime.clear();
    }
  }
  
  public synchronized TimeSet getTime()
    { return time; }
  
  /** Getter for property startMjd.
   * @return Value of property startMjd.
   */
  public double getStartMjd() {
    return getTime().getStartMjd();
  }
  /** Setter for property startMjd.
   * @param startMjd New value of property startMjd.
   *
   * @throws PropertyVetoException
   */
  public void setStartMjd(double startMjd) throws java.beans.PropertyVetoException {
    futureTime.setStartMjd(startMjd);
    setTime(futureTime);
  }
  
  /** Getter for property stopMjd.
   * @return Value of property stopMjd.
   */
  public double getStopMjd() {
    return getTime().getStopMjd();
  }
  /** Setter for property stopMjd.
   * @param stopMjd New value of property stopMjd.
   *
   * @throws PropertyVetoException
   */
  public void setStopMjd(double stopMjd) throws java.beans.PropertyVetoException {
    futureTime.setStopMjd(stopMjd);
    setTime(futureTime);
  }
  /** Getter for property stepMjd.
   * @return Value of property stepMjd.
   */
  public double getStepMjd() {
    return getTime().getStepMjd();
  }
  /** Setter for property stepMjd.
   * @param stepMjd New value of property stepMjd.
   *
   * @throws PropertyVetoException
   */
  public void setStepMjd(double stepMjd) throws java.beans.PropertyVetoException {
    futureTime.setStepMjd(stepMjd);
    setTime(futureTime);
  }
  
  /** Getter for property currentMjd.
   * @return Value of property currentMjd.
   */
  public double getCurrentMjd() {
    return getTime().getCurrentMjd();
  }
  /** Setter for property currentMjd.
   * @param currentMjd New value of property currentMjd.
   *
   * @throws PropertyVetoException
   */
  public void setCurrentMjd(double currentMjd) throws java.beans.PropertyVetoException {
    double oldCurrentMjd = getCurrentMjd();
    
    TimeSet oldTime = (TimeSet)getTime().clone();
    TimeSet newTime = (TimeSet)getTime().clone();
    newTime.setCurrentMjd(currentMjd);
    
    vetoableChangeSupport.fireVetoableChange(PROP_CURRENT_MJD, new Double (oldCurrentMjd), new Double (currentMjd));
    time.setCurrentMjd(currentMjd);
    propertySupport.firePropertyChange(PROP_CURRENT_MJD, new Double (oldCurrentMjd), new Double (currentMjd));
    
    timeChangeSupport.fireTimeChange(PROP_CURRENT_MJD, oldTime, newTime, getTimeMap());
  }
  
  public void vetoableChange(PropertyChangeEvent pche) throws java.beans.PropertyVetoException {
    if (pche.getPropertyName().equals(PROP_CURRENT_MJD)) {
      currentMjd = ((Double)(pche.getNewValue())).doubleValue();
      String reason = "";
      if (currentMjd > getStopMjd() + 2e-10 ) {
        reason = "Time is greater, than stop time\n(" +
                  Time.toString(currentMjd)+" > " + 
                  Time.toString(getStopMjd())+") ";// +currentMjd+" "+getStopMjd();
        throw new PropertyVetoException(reason, pche);
      }
      
      if (currentMjd < getStartMjd()) {
        reason = "Time is less, than start time\n(" +
                  Time.toString(currentMjd)+" > " + 
                  Time.toString(getStartMjd())+")";
        throw new PropertyVetoException(reason, pche);
      }
    }
    
    if (pche.getPropertyName().equals(PROP_TIME)) {
      TimeSet newTime = (TimeSet)(pche.getNewValue());
      String reason = "";
      if (newTime.getStartMjd() > newTime.getStopMjd()) {
        reason = "Start time could not be greater than stop time\n(" +
                  Time.toString(newTime.getStartMjd())+" > " + 
                  Time.toString(newTime.getStopMjd())+")";
        throw new PropertyVetoException(reason, pche);
      }
      
      if (newTime.getStepMjd() > Math.abs(newTime.getStartMjd() - newTime.getStopMjd())) {
        reason = "Time step could not be greater than\ntime interval between start and stop time";
        throw new PropertyVetoException(reason, pche);
      }
    
      // check number of intervals
      if ((newTime.getStopMjd() - newTime.getStartMjd()) / newTime.getStepMjd() > 200) {
        reason = "Number of steps exceeds 200";
        throw new PropertyVetoException(reason, pche);
      } 
      
    }
  }
 
  
  public String toString() 
    { return getName(); }
  
  public String getName() 
    { return "Time"; }
  

  public void setParent(Object parent) {
    this.parent = parent;
  }
  
  public Children getChildren() {
    return null;
  }
  
  //public OVTCore getCore();
  
  public void initialize() {
    propertyDescriptors = super.getDescriptors();
    propertyDescriptors.clear();
    try {
        // time
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("time", this);
        pd.setDisplayName("Time Settings");
        pd.setTextAccessible(false);
        
        GUIPropertyEditor editor = new TimeEditor(pd);
        // Render each time user changes time by means of gui
        editor.addGUIPropertyEditorListener(new GUIPropertyEditorListener() {
          public void editingFinished(GUIPropertyEditorEvent evt) {
            RenderAll();
          }
        });
        addPropertyChangeListener("time", editor);
        pd.setPropertyEditor(editor);
        propertyDescriptors.put(pd);
        
        // currentMjd
        pd = new BasicPropertyDescriptor("currentMjd", this);
        pd.setDisplayName("Current Time");
        pd.setLabel("Current Time");
        
        editor = new CurrentMjdEditor(pd);
        // Render each time user changes time by means of gui
        editor.addGUIPropertyEditorListener(new GUIPropertyEditorListener() {
          public void editingFinished(GUIPropertyEditorEvent evt) {
            RenderAll();
          }
        });
        
        addPropertyChangeListener("currentMjd", editor);
        addPropertyChangeListener("time", editor);
        
        pd.setPropertyEditor(editor);
        propertyDescriptors.put(pd);
        
        // startMjd
        pd = new BasicPropertyDescriptor("startMjd", this);
        pd.setDisplayName("Start Time");
        pd.setLabel("Start");
        pd.setTextOnlyAccessible();
        
        editor = new MjdEditor(pd);
        pd.setPropertyEditor(editor);
        propertyDescriptors.put(pd);
        
        // stopMjd
        pd = new BasicPropertyDescriptor("stopMjd", this);
        pd.setDisplayName("Stop Time");
        pd.setLabel("Stop");
        pd.setTextOnlyAccessible();
        
        editor = new MjdEditor(pd);
        pd.setPropertyEditor(editor);
        propertyDescriptors.put(pd);
        
        // stepMjd
        pd = new BasicPropertyDescriptor("stepMjd", this);
        pd.setDisplayName("Step [mm:ss]");
        pd.setLabel("Step");
        pd.setTextOnlyAccessible();
        
        editor = new MjdEditor(pd);
        pd.setPropertyEditor(editor);
        propertyDescriptors.put(pd);
        
    } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
        System.exit(0);
    }
  }
  
  private void setTimeMap(TimeSet ts) {
    timeMap.removeAllElements();
    double[] values = ts.getTimeMap();
    for (int index=0; index<values.length; index++)
      timeMap.addElement(new Double(values[index]));
  }
  
  public Vector getTimeMap() {
    if (timeMap.size() == 0) setTimeMap(getTime());
    return timeMap;
  }
  
    /** returns time, from wich is possible to start 
   *
   */
  public double getStartFor(double mjd) {
    double start = getStartMjd();
    double step = getStepMjd();
    if (mjd <= start) return start;
    else {
      // mjd > startMjd
      int n = (int)((mjd - start) / step);
      return start + step * (n + 1);
    }
  }
  
  
  public void listMethods() {
    Method[] methods = getClass().getMethods();
    System.out.println("Methods of " + getClass().getName()+ "-------------");
    for(int i=0; i<methods.length; i++)
      System.out.println("\t "+methods[i].toString());
    System.out.println("-------- end ---------");
  }
  
  
}
