/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/VisualObject.java,v $
  Date:      $Date: 2000/04/11 15:42:27 $
  Version:   $Revision: 1.14 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

/*
 * VisualObject.java
 *
 * Created on March 10, 2000, 6:27 PM
 */
 
package ovt.object;

import ovt.*;
import ovt.beans.*;
import ovt.interfaces.*;

import vtk.*;

import java.beans.*;
import javax.swing.*;
/** 
 * The Visual Object is an object, that deals with Visualization.
 * @author  root
 * @version 
 */
public class VisualObject extends BasicObject implements PropertyChangeListener {

  /** Holds value of property visible. */
  private boolean visible;

  /** Creates new VisualObject */
  public VisualObject(OVTCore core) {
    super(core);
  }
  
  /** Creates new VisualObject with name*/
  public VisualObject(OVTCore core, String name) {
    super(core, name);
  }
  
  /** Creates new VisualObject with name and Icon*/
  public VisualObject(OVTCore core, String name, String iconFilename) {
    super(core, name);
    setIcon(new ImageIcon(iconFilename));
  }
  
  public Descriptors getDescriptors() {
    if (descriptors == null) {
    // Add default property descriptor for visible property.
    // each visual object can be hidden or shown.
      try {
      
        descriptors = new Descriptors();
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("visible", this);
        pd.setDisplayName("Visibility");
        
        BasicPropertyEditor editor = new VisibilityEditor(pd);
        addPropertyChangeListener("visible", editor);
        addPropertyChangeListener(editor);
        pd.setPropertyEditor(editor);
        descriptors.put(pd);
      
      } catch (IntrospectionException e2) {
        System.out.println(getClass().getName() + " -> " + e2.toString());
        System.exit(0);
      }
    }
    return descriptors;
  }
  
  
  /** Getter for property visible.
   * @return Value of property visible.
   */
  public boolean isVisible() {
    return visible;
  }
  /** Setter for property visible.
   * @param visible New value of property visible.
   *
   * @throws PropertyVetoException
   */
  public void setVisible(boolean visible) {
    boolean oldVisible = this.visible;
    //if (oldVisible == visible) return; // nothing to change
    if (!isEnabled()) return; // object is not enabled - no motion. ;)
    this.visible = visible;
    firePropertyChange ("visible", new Boolean (oldVisible), new Boolean (visible));
  }
  
  
  
  public void setEnabled(boolean enabled) {
    if (isVisible()) setVisible(false);
    super.setEnabled(enabled);
  }
  
    /** Listens to events from parent */
  public void propertyChange(PropertyChangeEvent evt) {
    //System.out.println("Parent is " + getParent());
    //System.out.print("Is this event from my parent ... ");
    if (evt.getSource() == getParent()) { 
      // this is event from my parent
      //System.out.println("yes!");
      if (evt.getPropertyName().equals("enabled")) { // Sat.enabled property changed
        boolean satEnabled = ((Boolean)evt.getNewValue()).booleanValue();
        if (satEnabled != isEnabled())  setEnabled(satEnabled);
      }
    
      if (evt.getPropertyName().equals("visible")) { // Sat.visible property changed
        boolean visible = ((Boolean)evt.getNewValue()).booleanValue();
        if (visible != isVisible())  setVisible(visible);
      }
    } //else System.out.println("no :(");
  }
}