/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/object/editor/TimeEditorPanel.java,v $
  Date:      $Date: 2000/04/11 15:42:29 $
  Version:   $Revision: 1.4 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.object.editor;

import ovt.*;
import ovt.util.*;
import ovt.beans.*;
import ovt.datatype.*;


import java.beans.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class TimeEditorPanel extends JPanel implements PropertyChangeListener {

      private TimeSet timeSet;
      private TimeEditor editor;
      private MjdEditor startEditor, stopEditor;
      private MinutesEditor stepEditor;
      private JButton okButton, cancelButton;


public TimeEditorPanel(TimeEditor ed) {
  editor = ed;
  timeSet = (TimeSet)(((TimeSet)(editor.getValue())).clone());

  try {
        BasicPropertyDescriptor pd = new BasicPropertyDescriptor("startMjd", this);
        pd.setDisplayName("start");
        startEditor = new MjdEditor(pd);

        pd = new BasicPropertyDescriptor("stopMjd", this);
        pd.setDisplayName("stop");
        stopEditor = new MjdEditor(pd);

        pd = new BasicPropertyDescriptor("stepMjd", this);
        pd.setDisplayName("step [mm:ss]");
        stepEditor = new MinutesEditor(pd);

  } catch (IntrospectionException e2) { System.out.println(e2.toString()); }

  okButton = new JButton("Ok");

  okButton.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent ae) {
            try {
              editor.setTime(timeSet);
              editor.fireEditingFinished();
            } catch (PropertyVetoException e) {
              JOptionPane.showMessageDialog(getMe(), "" + e.getMessage(), "Input Error: PropertyVetoException", JOptionPane.WARNING_MESSAGE);
            }
          }
        });

  cancelButton = new JButton("Cancel");
  cancelButton.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent ae) {
            editor.fireEditingFinished();
          }
        });

  makeInterior();
}

  private Component getMe()
    { return this; }

  private void makeInterior() {

        JPanel container = new JPanel(false);
        container.setBorder(BorderFactory.createEmptyBorder(5,10,0,10));
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

        JPanel comp = new JPanel(false);
        comp.setLayout(new BoxLayout(comp, BoxLayout.Y_AXIS));
        comp.add(new JLabel("Start "));

        comp.add(startEditor.getComponent());


        comp.add(new JLabel("Stop "));
        comp.add(stopEditor.getComponent());

        comp.add(new JLabel("Tracing Step [mins]"));
        comp.add(stepEditor.getComponent());

        /*comp.add(new JLabel("Interval [hours] ", JLabel.CENTER));
        container.add(comp);

        comp = new JPanel(false);
        JButton back = new JButton("<<");
        back.setName("back");
        back.addActionListener(this);


        intervalField = new JTextField(Double.toString(calculateInterval()),9);
        intervalField.setName("interval");
        intervalField.addFocusListener(this);
        intervalField.addActionListener(this);

        JButton forward = new JButton(">>");
        forward.setName("forward");
        forward.addActionListener(this);

        comp.add(back);
        comp.add(intervalField);
        comp.add(forward);*/

        container.add(comp);

        comp = new JPanel();

        comp.setAlignmentX(LEFT_ALIGNMENT);
        comp.add(cancelButton);
        comp.add(okButton);

        container.add(comp);
        add(container);

      }

      // Let's override parent's method to update the window each time
      // it is shown
      /*public void setVisible(boolean visible) {
      if (visible) {
      getContentPane().removeAll();
      makeInterior();
      pack();
      }
      super.setVisible(visible);
      }*/


public void propertyChange(PropertyChangeEvent pche) {
  timeSet = (TimeSet)(editor.getValue());
  if (pche.getPropertyName().equals(editor.getPropertyName())) {
          // tell the Editors, that life's changing ;)
          startEditor.propertyChange(pche);
          stopEditor.propertyChange(pche);
          stepEditor.propertyChange(pche);
  }
}


  public void setStartMjd(double value) throws PropertyVetoException {
    timeSet.setStartMjd(value);
  }

  public void setStopMjd(double value) throws PropertyVetoException  {
        timeSet.setStopMjd(value);
  }

  public void setStepMjd(double value) throws PropertyVetoException  {
        timeSet.setStepMjd(value);
  }

  public double getStartMjd() {
        return timeSet.getStartMjd();
  }

  public double getStopMjd() {
        return timeSet.getStopMjd();
  }

  public double getStepMjd() {
        return timeSet.getStepMjd();
  }


}
