/*=========================================================================

  Program:   Orbit Visualization Tool
  Source:    $Source: /net/nest/stor/devel/ovt2g/ovt/util/Vect.java,v $
  Date:      $Date: 2000/06/05 12:15:02 $
  Version:   $Revision: 1.9 $


Copyright (c) 2000 OVT Team (Kristof Stasiewicz, Mykola Khotyaintsev, 
Yuri Khotyaintsev)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification is permitted provided that the following conditions are met:

 * No part of the software can be included in any commercial package without
written consent from the OVT team.

 * Redistributions of the source or binary code must retain the above
copyright notice, this list of conditions and the following disclaimer.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT OR
INDIRECT DAMAGES  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.

OVT Team (http://ovt.irfu.se)   K. Stasiewicz, M. Khotyaintsev, Y.
Khotyaintsev

=========================================================================*/

package ovt.util;

import java.util.*;

public class Vect {

  public static double[] add(double[] vector1, double[] vector2) {
    double[] res = new double[vector1.length];
    for (int i=0; i<vector1.length; i++)
      res[i] = vector1[i] + vector2[i];
    return res;
  }
  
  //added by kono
  public static double[] sub(double[] vector1, double[] vector2) {
    double[] res = new double[vector1.length];
    for (int i=0; i<vector1.length; i++)
      res[i] = vector1[i] - vector2[i];
    return res;
  }
  
  public static double angleOf2vect(double[] v1,double[] v2){
     double abs1x2=absv(v1)*absv(v2);
     if(abs1x2==0.0)
        return 0.5*Math.PI;
     double[] v3=cross(v1,v2);
     double dotprd=dot(v1,v2); //v3 = v1 x v2
     return Math.atan2(absv(v3),dotprd/abs1x2);
  }
  
  public static double[] multiply(double[] vector, double v) {
    double[] res = new double[vector.length];
    for (int i=0; i<vector.length; i++)
      res[i] = vector[i] * v;
    return res;
  }
  
public static double absv(double vec[]) {

	double sum = 0;

    for (int i=0; i<3; i++)
        sum += vec[i]*vec[i];
		
    return Math.sqrt(sum);
}

public static double absv2(double vec[]) {

	double sum = 0;

    for (int i=0; i<3; i++)
        sum += vec[i]*vec[i];
		
    return sum;
}


/* change length of vector vec(3) to norm */

public static void normf(double vec[], double norm) {

    double s;

    /* Function Body */

    s = absv(vec);

    /*for (d1 = vec; d1 < vec + 3; ++d1)
        *d1 *= norm / s;*/
	for (int i=0; i<3; i++)
		vec[i] *= norm / s;

}


/* scalar product of two vectors */

public static double  dot(double a[], double b[]) {

    double  ret_val = 0.0;

	for(int i=0; i<3; i++)
		ret_val += a[i] * b[i]; 

    return (ret_val);
}

  /** vector product c =  a x b */
  public static double[] cross(double a[], double b[]) {
    double c[] = new double[3];
    c[0] = a[1] * b[2] - a[2] * b[1];
    c[1] = a[2] * b[0] - a[0] * b[2];
    c[2] = a[0] * b[1] - a[1] * b[0];
    return c;
  }


/** vector product c =  a x b 
 * @depricated since 0.0001
 */
public static void cross(double a[], double b[], double c[]) {
	
	c[0] = a[1] * b[2] - a[2] * b[1];
    c[1] = a[2] * b[0] - a[0] * b[2];
    c[2] = a[0] * b[1] - a[1] * b[0];
	
}


/** normalized vector product */
public static double[] crossn(double a[], double b[]) {
  double c[] = new double[3];
  double x;
  cross(a, b, c);
  x = absv(c);

    if (x >= 1e-30)
  for (int i=0; i<3; i++) 
	c[i] = c[i] / x;
    return c;
}


// Looks like - made for sequence
public static void crossn(double a[], double b[], double c[]) {

    double x;

    cross(a, b, c);
    x = absv(c);

    if (x >= 1e-30)
		for (int i=0; i<3; i++) 
			c[i] = c[i] / x;

  }

  /** Returns the array of Vector's elements. Used for JAK1.1 compatibility.
   * JDK 1.1 has no Vector.toArray() method.
   */
  public static Object[] toArray(Vector vect) {
    if (vect.size() == 0) return null;
    Object[] res = new Object[vect.size()];
    int i=0;
    Enumeration e = vect.elements();
    while(e.hasMoreElements()) {
      //System.out.println(i + "\t[" + vect.size() + "]");
      res[i] = e.nextElement();
      i++;
    }
    return res;
  }
  
  public static void main(String[] x){
    //double[] v1={-200.0,300.0,-50.0},v2={100.0,-90.0,80.0};
    double[] v1={0.0,0.0,15.0},v2={0.0,0.0,-100.0};
    System.out.println("Angle: "+angleOf2vect(v1,v2)*180.0/Math.PI+" deg");
  }

}
